local LibDeflate = LibStub:GetLibrary("LibDeflate")

function MorthalinRaidTools:OnCommReceived(prefix, message, distribution, sender)
	local senderData = MorthalinRaidTools:GetPlayerInfo(sender);
	local success = false;
	local data;
	if(MorthalinRaidTools.db.global.allowBroadcastIn and senderData ~= nil and senderData.rankIndex <= MorthalinRaidTools.db.global.broadcastIncomingRank) then
		local decoded;
		if(prefix == "MRTHistory") then
			decoded = LibDeflate:DecompressDeflate(LibDeflate:DecodeForWoWAddonChannel(message));
		else
			decoded = message;
		end
		success, data = self:Deserialize(decoded);
		if(success) then
			if(sender ~= UnitName("player")) then
				if(prefix == "MRTAddBench") then -- add player on bench
					self:AddBenchFromBroadcast(data, sender);
				elseif(prefix == "MRTRemoveBench") then -- remove player from bench
					self:RemoveBenchFromBroadcast(data, sender);
				elseif(prefix == "MRTFullBench") then -- get whole bench
					if(type(data) ~= "table") then
						self:ResetList(false);
					else
						for i = 1, #data do
							self:AddBenchFromBroadcast(data[i], sender);
						end
					end
				elseif(prefix == "MRTEmptyBench") then -- remove all from bench
					self:ResetList(true);
				elseif(prefix == "MRTKill") then -- add kill
					local found = false;
					if(GetRaidRosterInfo(1) ~= null) then
						for i = 1, 40 do
							local name, rank, subgroup, level, class = GetRaidRosterInfo(i);
							if(name == sender) then
								found = true;
							end
						end
					end
					if(found == false) then
						self:AddKillFromBroadcast(data, sender);
						MorthalinRaidTools:LoadStatistics();
						MorthalinRaidTools:UpdateHistory();
						MorthalinRaidTools:GetZones();
					end
				elseif(prefix == "MRTHistory") then -- merge history
					for i = 1, #data do
						self:AddKillFromBroadcast(data[i], sender, true);
					end
					MorthalinRaidTools:GetZones();
					MorthalinRaidTools:UpdateHistory();
					MorthalinRaidTools:Print("History data was received by " .. sender ..  ".");
				elseif(prefix == "MRTAddHistPlayer") then -- add player or kill to history
					MorthalinRaidTools:AddPlayerToKillFromBroadcast(data, sender);
					MorthalinRaidTools:UpdateHistory();
				elseif(prefix == "MRTRemoveKill") then -- remove kill from history
					MorthalinRaidTools:RemoveKillFromBroadcast(data, sender);
					MorthalinRaidTools:UpdateHistory();
				elseif(prefix == "MRTAskForBench" and MorthalinRaidTools.db.global.mainBroadcaster) then
					MorthalinRaidTools:BroadcastBench(sender);
				elseif(prefix == "MRTAskForHist7" and MorthalinRaidTools.db.global.mainBroadcaster) then
					MorthalinRaidTools:BroadcastHistory7(sender);
				elseif(prefix == "MRTBuffSettings") then
					MorthalinRaidTools:BroadcastBuffSettings(data, sender);
				elseif(prefix == "MRTItemSettings") then
					MorthalinRaidTools:BroadcastItemSettings(data, sender);
				elseif(prefix == "MRTDebuffSet") then
					MorthalinRaidTools:BroadcastDebuffSettings(data, sender);
				elseif(prefix == "MRTDebuffSetSel") then
					MorthalinRaidTools:BroadcastDebuffSettingsSelected(data, sender);
				end
			end
		else
			DEFAULT_CHAT_FRAME:AddMessage("|cffff0000" .. data);
		end
	elseif(prefix == "MRTVersionCheck") then
		success, data = self:Deserialize(message);
		if(success) then
			if(sender ~= UnitName("player")) then
				MorthalinRaidTools:BroadcastVersionCheck(data);
			end
		end
	end
	if(success and sender == UnitName("player")) then
		if(prefix == "MRTHistory") then
			MorthalinRaidTools:Print("History data was sent to guild members.");
		end
	end
end

function MorthalinRaidTools:BroadcastAddBench(player)
	if(MorthalinRaidTools.db.global.allowBroadcastOut) then
		local serialized = self:Serialize(player);
		self:SendCommMessage("MRTAddBench", serialized, "GUILD");
	end
end

function MorthalinRaidTools:AddBenchFromBroadcast(player, addedBy)
	local found = false;
	table.foreach(self.db.global.bench.players, function(index, value) 
		if(value.name == player.name) then 
			self:Print("|cffff0000Player |cff" .. value.color .. player.name .. "|cffff0000 is already on bench.");
			found = true;
		end 
	end)
	if(found == false) then
		table.insert(self.db.global.bench.players, player);
		self:Print(addedBy .. " added |cff" .. self.db.global.bench.players[#self.db.global.bench.players].color ..  player.name .. "|r on bench.");
		MorthalinRaidTools:UpdateBenchView();
	end
end

function MorthalinRaidTools:BroadcastRemoveBench(player)
	if(MorthalinRaidTools.db.global.allowBroadcastOut) then
		local serialized = self:Serialize(player);
		self:SendCommMessage("MRTRemoveBench", serialized, "GUILD");
	end
end

function MorthalinRaidTools:RemoveBenchFromBroadcast(player, removedBy)
	local found = false;
	table.foreach(self.db.global.bench.players, function(index, value) 
		if(value.name == player.name) then 
			table.remove(self.db.global.bench.players, index);
			self:Print("Player |cff" .. value.color .. player.name .. "|r was removed from bench by " .. removedBy .. ".");
			MorthalinRaidTools:UpdateBenchView();
			found = true;
		end 
	end)
	if(found == false) then
		self:Print("|cffff0000Player " .. player.name .. " is not on bench.");
	end
end

function MorthalinRaidTools:BroadcastBench(requester)
	if(MorthalinRaidTools.db.global.allowBroadcastOut) then
		local serialized;
		if(#self.db.global.bench.players == 0) then
			serialized = self:Serialize("empty");
		else
			serialized = self:Serialize(self.db.global.bench.players);
		end
		self:SendCommMessage("MRTFullBench", serialized, "GUILD");
		local guild,_,_ = GetGuildInfo(UnitName("player"));
		local realm = GetRealmName();
		if(guild == "Deep Thunder" and realm == "Golemagg") then
			MorthalinRaidTools:BroadcastBenchDKP();
		end
	end
end

function MorthalinRaidTools:AskForBenchBroadcast()
	local serialized = self:Serialize("MRTAskForBench");
	self:SendCommMessage("MRTAskForBench", serialized, "GUILD");
end

function MorthalinRaidTools:AskForHistory7Broadcast()
	local serialized = self:Serialize("MRTAskForHist7");
	self:SendCommMessage("MRTAskForHist7", serialized, "GUILD");
end

function MorthalinRaidTools:BroadcastEmptyBench()
	if(MorthalinRaidTools.db.global.allowBroadcastOut) then
		local serialized = self:Serialize("true");
		self:SendCommMessage("MRTEmptyBench", serialized, "GUILD");
	end
end

function MorthalinRaidTools:BroadcastKill(kill)
	if(MorthalinRaidTools.db.global.allowBroadcastOut) then
		local serialized = self:Serialize(kill);
		self:SendCommMessage("MRTKill", serialized, "GUILD");
	end
end

function MorthalinRaidTools:BroadcastKillFromHistory(kill)
	if(MorthalinRaidTools.db.global.allowBroadcastOut) then
		local selected = {};
		table.insert(selected, kill);
		MorthalinRaidTools:Print("Sending history data...");
		local serialized = MorthalinRaidTools:Serialize(selected);
		local compressed = LibDeflate:CompressDeflate(serialized, {level = 6});
		local packet;
		if compressed then
			packet = LibDeflate:EncodeForWoWAddonChannel(compressed);
		end
		MorthalinRaidTools:SendCommMessage("MRTHistory", packet, "GUILD");
	end
end

function MorthalinRaidTools:BroadcastInstanceFromHistory(instance)
	if(MorthalinRaidTools.db.global.allowBroadcastOut) then
		local selected = {};
		local count = #self.db.global.history;
		for i = 1, count do
			if (date("%H", self.db.global.history[(count + 1 - i)].killDate) == "00" or date("%H", self.db.global.history[(count + 1 - i)].killDate) == "01") then
				dateTemp = date("%d.%m.%Y",(self.db.global.history[count + 1 - i].killDate - 7200))
			else
				dateTemp = date("%d.%m.%Y",self.db.global.history[count + 1 - i].killDate)
			end
			if (dateTemp == date("%d.%m.%Y",instance.start) and self.db.global.history[count + 1 - i].zone == instance.name) then
				table.insert(selected, self.db.global.history[count + 1 - i]);
			end
		end
		MorthalinRaidTools:Print("Sending history data...");
		local serialized = MorthalinRaidTools:Serialize(selected);
		local compressed = LibDeflate:CompressDeflate(serialized, {level = 6});
		local packet;
		if compressed then
			packet = LibDeflate:EncodeForWoWAddonChannel(compressed);
		end
		MorthalinRaidTools:SendCommMessage("MRTHistory", packet, "GUILD");
	end
end

function MorthalinRaidTools:BroadcastDateFromHistory(historyDate)
	if(MorthalinRaidTools.db.global.allowBroadcastOut) then
		local selected = {};
		local count = #self.db.global.history;
		for i = 1, count do
			if (date("%H", self.db.global.history[(count + 1 - i)].killDate) == "00" or date("%H", self.db.global.history[(count + 1 - i)].killDate) == "01") then
				dateTemp = date("%d.%m.%Y",(self.db.global.history[count + 1 - i].killDate - 7200))
			else
				dateTemp = date("%d.%m.%Y",self.db.global.history[count + 1 - i].killDate)
			end
			if (dateTemp == historyDate) then
				table.insert(selected, self.db.global.history[count + 1 - i]);
			end
		end
		MorthalinRaidTools:Print("Sending history data...");
		local serialized = MorthalinRaidTools:Serialize(selected);
		local compressed = LibDeflate:CompressDeflate(serialized, {level = 6});
		local packet;
		if compressed then
			packet = LibDeflate:EncodeForWoWAddonChannel(compressed);
		end
		MorthalinRaidTools:SendCommMessage("MRTHistory", packet, "GUILD");
	end
end

function MorthalinRaidTools:BroadcastHistory()
	if(MorthalinRaidTools.db.global.allowBroadcastOut) then
		MorthalinRaidTools:Print("Sending history data...");
		local serialized = MorthalinRaidTools:Serialize(self.db.global.history);
		local compressed = LibDeflate:CompressDeflate(serialized, {level = 6});
		local packet;
		if compressed then
			packet = LibDeflate:EncodeForWoWAddonChannel(compressed);
		end
		MorthalinRaidTools:SendCommMessage("MRTHistory", packet, "GUILD");
	end
end

function MorthalinRaidTools:BroadcastHistory7(requester)
	MorthalinRaidTools:Print("Sending history data for " .. requester);
	local selected = {};
	local looping = true;
	local i = #self.db.global.history;
	while looping do
		if(self.db.global.history[i].killDate > (GetServerTime() - 604800) and i > 0) then
			table.insert(selected,self.db.global.history[i]);
		else
			looping = false;
		end
		i = i - 1;
	end
	
	local serialized = MorthalinRaidTools:Serialize(selected);
	local compressed = LibDeflate:CompressDeflate(serialized, {level = 6});
	local packet;
	if compressed then
		packet = LibDeflate:EncodeForWoWAddonChannel(compressed);
	end
	MorthalinRaidTools:SendCommMessage("MRTHistory", packet, "GUILD");
end

function MorthalinRaidTools:AddKillFromBroadcast(kill, sender, merge)
	local found = false;
	for i = 1, #self.db.global.history do
		if(self.db.global.history[i].bossID == kill.bossID and self.db.global.history[i].instanceID == kill.instanceID) then
			found = true;
			if(merge ~= nil) then
				if(MorthalinRaidTools.db.global.allowBroadcastChange) then
					self.db.global.history[i].members = kill.members;
				end
			end
		end
	end
	if(found == false) then
		table.insert(self.db.global.history, kill);
		if(merge == nil) then
			self:Print("Guild killed " .. kill.bossName .. " in " .. kill.zone .. ". Sent by " .. sender .. ".");
		end
	end
end

function MorthalinRaidTools:BroadcastHistoryAdd(kill)
	if(MorthalinRaidTools.db.global.allowBroadcastOut) then
		local serialized = self:Serialize(kill);
		self:SendCommMessage("MRTAddHistPlayer", serialized, "GUILD");
	end
end

function MorthalinRaidTools:AddPlayerToKillFromBroadcast(kill, sender)
	local index = 0;
	for i = 1, #self.db.global.history do
		if(self.db.global.history[i].bossID == kill.bossID and self.db.global.history[i].instanceID == kill.instanceID and self.db.global.history[i].resetDate == kill.resetDate) then
			index = i;
		end
	end
	
	if(index == 0) then
		table.insert(self.db.global.history, data);
		self:Print(sender .. " added missing " .. kill.bossName .. " kill from " .. date("%d.%m.%Y",kill.killDate) .. " to history.");
	elseif(#self.db.global.history[index].members <= #kill.members or self.db.global.allowBroadcastRemove) then
		self.db.global.history[index].members = kill.members;
		self:Print(sender .. " updated members on " .. kill.bossName .. "kill from " .. date("%d.%m.%Y",kill.killDate) .. ".");
	end
end

function MorthalinRaidTools:BroadcastHistoryRemove(kill)
	if(MorthalinRaidTools.db.global.allowBroadcastOut) then
		local serialized = self:Serialize(kill);
		self:SendCommMessage("MRTAddHistPlayer", serialized, "GUILD");
	end
end

function MorthalinRaidTools:RemoveKillFromBroadcast(kill, sender)
	local index = 0;
	for i = 1, #self.db.global.history do
		if(self.db.global.history[i].bossID == kill.bossID and self.db.global.history[i].instanceID == kill.instanceID) then
			index = i;
		end
	end
	
	if(index ~= 0 and self.db.global.allowBroadcastRemove) then
		table.remove(self.db.global.history, index);
		self:Print(sender .. " removed " .. kill.bossName .. "kill from " .. date("%d.%m.%Y",kill.killDate) .. ".");
	end
end

function MorthalinRaidTools:SendBuffSettings(target)
	local serialized = self:Serialize(self.db.global.buffs);
	self:SendCommMessage("MRTBuffSettings", serialized, "WHISPER", target);
	MorthalinRaidTools:Print("Buff settings sent to " .. target .. ".");
end

function MorthalinRaidTools:BroadcastBuffSettings(data, sender)
	local buffs = {};
	for i = 1, #self.db.global.buffs do
		if(buffs[self.db.global.buffs[i].name] == nil) then
			buffs[self.db.global.buffs[i].name] = self.db.global.buffs[i].name;
		end
	end
	for i = 1, #data do
		if(buffs[data[i].name] == nil) then
			table.insert(self.db.global.buffs, data[i]);
		end
	end
	self:Print(sender .. " sent you buffs settings.");
end

function MorthalinRaidTools:SendDebuffSettings()
	local serialized = self:Serialize(self.db.global.timedAuras);
	self:SendCommMessage("MRTDebuffSet", serialized, "GUILD");
	MorthalinRaidTools:Print("Debuffs settings sent to guild members.");
end

function MorthalinRaidTools:BroadcastDebuffSettings(data, sender)
	for name, dataD in pairs(data) do
		if(MorthalinRaidTools.db.global.timedAuras[name] == nil) then
			MorthalinRaidTools.db.global.timedAuras[name] = dataD;
		end
	end
	self:Print(sender .. " sent you debuffs settings.");
end

function MorthalinRaidTools:SendDebuffSettingsSelected(name)
	local array = {};
	table.insert(array, name);
	table.insert(array, self.db.global.timedAuras[name].count);
	local serialized = self:Serialize(array);
	self:SendCommMessage("MRTDebuffSetSel", serialized, "GUILD");
	MorthalinRaidTools:Print("Debuff settings sent to guild members.");
end

function MorthalinRaidTools:BroadcastDebuffSettingsSelected(data, sender)
	local name = data[1];
	local dataD = data[2];
	if(MorthalinRaidTools.db.global.timedAuras[name] == nil) then
		MorthalinRaidTools.db.global.timedAuras[name] = {count = dataD};
	end
	self:Print(sender .. " sent you debuff settings.");
end

function MorthalinRaidTools:SendItemSettings()
	if(MorthalinRaidTools.db.global.allowBroadcastOut) then
		local serialized = self:Serialize(self.db.global.itemCounter);
		self:SendCommMessage("MRTItemSettings", serialized, "GUILD");
		MorthalinRaidTools:Print("Item settings data sent.");
	end
end

function MorthalinRaidTools:BroadcastItemSettings(data, sender)
	self.db.global.itemCounter = data;
	self:Print(sender .. " sent you item settings data.");
end

function MorthalinRaidTools:BroadcastAddBenchDKP(playerName)
	local serialized = self:Serialize(playerName);
	self:SendCommMessage("CommDKPStandAd", serialized, "GUILD");
end

function MorthalinRaidTools:BroadcastRemoveBenchDKP(playerName)
	local serialized = self:Serialize(playerName);
	self:SendCommMessage("CommDKPStandRm", serialized, "GUILD");
end

function MorthalinRaidTools:BroadcastClearBenchDKP()
	local serialized = self:Serialize("CommDKPStandCl");
	self:SendCommMessage("CommDKPStandCl", serialized, "GUILD");
end

function MorthalinRaidTools:BroadcastBenchDKP()
	local serialized = self:Serialize("CommDKPStandRf");
	self:SendCommMessage("CommDKPStandRf", serialized, "GUILD");
end

function MorthalinRaidTools:SendBroadcastVersionCheck()
	local serialized = self:Serialize(GetAddOnMetadata("MorthalinRaidTools", "Version"));
	self:SendCommMessage("MRTVersionCheck", serialized, "GUILD");
end

function MorthalinRaidTools:BroadcastVersionCheck(version)
	if(tonumber(GetAddOnMetadata("MorthalinRaidTools", "Version")) < tonumber(version) and MorthalinRaidTools.versionWarrning == false) then
		MorthalinRaidTools:Print("Newer version is availible!");
		MorthalinRaidTools.versionWarrning = true;
	end
end
