local testData, core = ...;
--COLORS OF CLASSES
MorthalinRaidTools.classColors = {
	["Druid"] = { r = 1, g = 0.49, b = 0.04, hex = "FF7D0A" },
	["Hunter"] = {  r = 0.67, g = 0.83, b = 0.45, hex = "ABD473" },
	["Mage"] = { r = 0.25, g = 0.78, b = 0.92, hex = "40C7EB" },
	["Priest"] = { r = 1, g = 1, b = 1, hex = "FFFFFF" },
	["Rogue"] = { r = 1, g = 0.96, b = 0.41, hex = "FFF569" },
	["Shaman"] = { r = 0, g = 0.44, b = 0.87, hex = "0070DD" },
	["Paladin"] = { r = 0.96, g = 0.55, b = 0.73, hex = "F58CBA" },
	["Warlock"] = { r = 0.53, g = 0.53, b = 0.93, hex = "8787ED" },
	["Warrior"] = { r = 0.78, g = 0.61, b = 0.43, hex = "C79C6E" },
	["Unknown"] = { r = 1, g = 0, b = 0, hex = "FF0000" }
}

function MorthalinRaidTools:GetClassColor(class)
	if(MorthalinRaidTools.classColors[class] == nil) then
		return MorthalinRaidTools.classColors["Unknown"].hex;
	else
		return MorthalinRaidTools.classColors[class].hex;
	end
end

function MorthalinRaidTools:AllowChat()
	if self.db.global.acceptBench then
		-- disable chat functions
		self.db.global.acceptBench = false;
		if(MorthalinRaidTools.autoInvSchedule == false) then
			MorthalinRaidTools:UnregisterEvent("CHAT_MSG_GUILD");
			MorthalinRaidTools:UnregisterEvent("CHAT_MSG_WHISPER");
			MorthalinRaidTools:UnregisterEvent("CHAT_MSG_BN_WHISPER");
		end
		self:Print("|cffff0000Addon stopped accepting raid members.")
	else
		-- enable chat functions
		self.db.global.acceptBench = true;
		if(MorthalinRaidTools.autoInvSchedule == false) then
			MorthalinRaidTools:RegisterEvent("CHAT_MSG_GUILD", function(...) MorthalinRaidTools:ChatChannelCallback(...) end);
			MorthalinRaidTools:RegisterEvent("CHAT_MSG_WHISPER", function(...) MorthalinRaidTools:ChatChannelCallback(...) end);
	    MorthalinRaidTools:RegisterEvent("CHAT_MSG_BN_WHISPER", function(...) MorthalinRaidTools:ChatChannelCallback(...) end);
		end
		self:Print("|cff00ff00Addon accepting raid members.")
	end
end

-- Function on command /mg bench "player name"
-- =========================
function MorthalinRaidTools:AddOnBench(playerName, delegate)
	local players = self.db.global.bench.players;
	local added = false;
	local found = false;
	playerName = string.upper(string.utf8sub(playerName, 1, 1))..string.lower(string.utf8sub(playerName, 2));
	
	if(MorthalinRaidTools.db.global.mainBenchInv) then
		for i=1, GetNumGuildMembers() do
			local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i);
			if(string.sub(name, 1, name:find("-") - 1) == playerName) then
				if(string.sub(officernote, 1, 4) == "Alt:") then
					playerName = string.sub(officernote, 5, -1);
				end
			end
		end
	end
	
	table.foreach(players, function(index, value) 
		if(value.name == playerName) then 
			self:Print("|cffff0000Player |cff" .. value.color .. playerName .. "|cffff0000 is already in list.");
			found = true;
		end 
	end)
	
	if(found == false) then
		local numTotalMembers = GetNumGuildMembers();
		local realmName = GetRealmName();
		local playerTemp = {
			name = {},
			class = {},
			color = {},
			group = {}
		}
		
		for i=1, numTotalMembers do
			local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i);
			if(name == playerName .. "-" .. realmName) then
				table.insert(players, playerTemp);
				players[#players].name = playerName;
				players[#players].class = class;
				players[#players].color = MorthalinRaidTools.classColors[class].hex;
				players[#players].group = 0;
				self:Print("|cff" .. players[#players].color .. playerName .. "|r added on bench.");
				if(delegate ~= nil) then
					SendChatMessage("You are on bench with nick " .. playerName .. ".", "WHISPER", "Common", delegate);
				end
				added = true;
				MorthalinRaidTools:BroadcastAddBench(players[#players]);
			end
		end
		
		if(added == false) then
			self:Print("|cffff0000Player " .. playerName .. " is not in guild or not exist.");
		else
			local guild,_,_ = GetGuildInfo(UnitName("player"));
			local realm = GetRealmName();
			if(guild == "Deep Thunder" and realm == "Golemagg") then
				MorthalinRaidTools:BroadcastAddBenchDKP(playerName);
			end
		end
		
		self:UpdateBenchView();
	end
end

function MorthalinRaidTools:CheckBenchInRaid()
	if(IsInRaid()) then
		for i = 1, 40 do
			local name = GetRaidRosterInfo(i);
			if(name ~= nil) then
				local playerInfo = MorthalinRaidTools:GetPlayerInfo(name);
				for j = 1, #self.db.global.bench.players do
					if(self.db.global.bench.players[j].name == name or (playerInfo ~= nil and playerInfo.officernote:sub(1, 4) == "Alt:" and self.db.global.bench.players[j].name == playerInfo.officernote:sub(5))) then
						self:Print("Player |cff" .. self.db.global.bench.players[j].color .. name .. "|r was moved to raid.");
						if(MorthalinRaidTools.db.global.mainBroadcaster) then
							SendChatMessage("You was removed from bench. If you leave raid remember write !bench again.", "WHISPER", "Common", name);
						end
						local guild,_,_ = GetGuildInfo(UnitName("player"));
						local realm = GetRealmName();
						if(guild == "Deep Thunder" and realm == "Golemagg") then
							MorthalinRaidTools:BroadcastRemoveBenchDKP(self.db.global.bench.players[j].name);
						end
						table.remove(self.db.global.bench.players, j);
						self:UpdateBenchView();
						return;
					end
				end
			end
		end
	end
end

-- Function on command /mg rm "player name"
-- =========================
function MorthalinRaidTools:RemoveFromBench(playerName)
	local players = self.db.global.bench.players;
	local removed = false;
	playerName = string.upper(string.utf8sub(playerName, 1, 1))..string.lower(string.utf8sub(playerName, 2));
	table.foreach(players, function(index, value) 
		if(value.name == playerName) then 
			MorthalinRaidTools:BroadcastRemoveBench(value);
			table.remove(players, index);
			removed = true;
			self:Print("Player |cff" .. value.color .. playerName .. "|r was removed from bench.");
		end 
	end)
	if(removed == false) then
		self:Print("|cffff0000Player " .. playerName .. " is not on bench.");
	else
		local guild,_,_ = GetGuildInfo(UnitName("player"));
		local realm = GetRealmName();
		if(guild == "Deep Thunder" and realm == "Golemagg") then
			MorthalinRaidTools:BroadcastRemoveBenchDKP(playerName);
		end
	end
	
	self:UpdateBenchView();
end

-- Function on command /mg list
-- =========================
function MorthalinRaidTools:GetList()
	if(#self.db.global.bench.players ~= 0) then
		self:Print("Players on bench:");
		for index, player in pairs(self.db.global.bench.players) do
			self:Print("|cff" .. player.color .. player.name);
		end
	else
		self:Print("|cffff0000List is empty!");
	end
end

-- Function on command /mg history
-- =========================
function MorthalinRaidTools:ShowHistory()
	self:Print("Kills:");
	for index, player in pairs(self.db.global.history) do
		self:Print(date("%d.%m.%Y %H:%M:%S",player.killDate) .. " - " .. player.bossName);
	end
end

-- Function on command /mg reset
-- =========================
function MorthalinRaidTools:ResetList(broadcast)
	if(broadcast == nil) then
		self:BroadcastEmptyBench();
	end
	local guild,_,_ = GetGuildInfo(UnitName("player"));
	local realm = GetRealmName();
	if(guild == "Deep Thunder" and realm == "Golemagg") then
		MorthalinRaidTools:BroadcastClearBenchDKP();
	end
	self.db.global.bench.players = {};
	self:Print("Bench is cleared.");
	self:UpdateBenchView();
end

function MorthalinRaidTools:AddBossKill(encounterID, encounterName, wipe, killTime)
	local instID = 1;
	if(encounterName == "Onyxia") then
		instID = tonumber(date("%d%m%y%H",GetServerTime()));
	end
	local resetSec;
	local encounterZone = MorthalinRaidTools.db.global.encounterList[encounterName].instance;
	RequestRaidInfo();
	local savedInstances = GetNumSavedInstances();
	for i = 1, savedInstances do
		local name, id, reset = GetSavedInstanceInfo(i);
		if(encounterZone == name) then
			instID = id;
			resetSec = reset;
		end
	end
	local guildMembers = 0;
	local raidMembers = 0;
	
	local timesheet = {
		bossID = encounterID,
		bossName = encounterName,
		pullDate = MorthalinRaidTools.db.global.buffsSnapshotTime,
		killDate = killTime,
		zone = encounterZone,
		instanceID = instID,
		resetDate = resetSec ~= nil and date("%d.%m.%Y",GetServerTime() + resetSec) or MorthalinRaidTools:SetResetDate(GetServerTime(), encounterZone),
		members = {},
		debuffs = MorthalinRaidTools.db.global.debuffs,
		wipe = wipe
	}
	
	MorthalinRaidTools:SetResetGlobal(timesheet.resetDate, timesheet.zone);
	
	for i=1, 40 do
		local playerTemp = {
			name = {},
			class = {},
			color = {},
			group = {},
			buffs = {}
		}
		local name, rank, subgroup, level, class, fileName, zone, online = GetRaidRosterInfo(i);
		
		if(name ~= nil) then
			table.insert(timesheet.members, playerTemp);
			timesheet.members[#timesheet.members].name = name;
			timesheet.members[#timesheet.members].class = class;
			timesheet.members[#timesheet.members].color = MorthalinRaidTools.classColors[class].hex;
			timesheet.members[#timesheet.members].group = subgroup;
			if(MorthalinRaidTools.db.global.members[name] ~= nil) then
				timesheet.members[#timesheet.members].buffs = MorthalinRaidTools.db.global.members[name].buffs;
			end
			if MorthalinRaidTools:IsGuildMember(name) then
				guildMembers = guildMembers + 1;
			end
			raidMembers = raidMembers + 1;
		else
			break;
		end
	end
	
	self:CheckBenchInRaid();
	
	if(#self.db.global.bench.players ~= 0) then
		for index, player in pairs(self.db.global.bench.players) do
			local playerTemp = {
				name = {},
				class = {},
				color = {},
				group = {}
			}
			table.insert(timesheet.members, playerTemp);
			timesheet.members[#timesheet.members].name = player.name;
			timesheet.members[#timesheet.members].class = player.class;
			timesheet.members[#timesheet.members].color = player.color;
			timesheet.members[#timesheet.members].group = player.group;
			timesheet.members[#timesheet.members].buffs = {};
		end
	end
	
	MorthalinRaidTools:ScheduleTimer(function()
		RequestRaidInfo();
		local savedInstances = GetNumSavedInstances();
		for i = 1, savedInstances do
			local name, id, reset = GetSavedInstanceInfo(i);
			if(encounterZone == name) then
				instID = id;
			end
		end
		timesheet.instanceID = instID;
		if(self.db.global.history ~= nil and #self.db.global.history ~= 0 and self.db.global.history[#self.db.global.history].resetDate == timesheet.resetDate and self.db.global.history[#self.db.global.history].instanceID == 1) then
			self.db.global.history[#self.db.global.history].instanceID = timesheet.instanceID;
		end
		
		if (self.db.global.allowOnlyGuildKills == false or (guildMembers * 100 / raidMembers) >= 50) then
			table.insert(self.db.global.history, timesheet);
		end
		if (self.db.global.allowOnlyGuildBroadcast == false or (guildMembers * 100 / raidMembers) >= 50) then 
			self:BroadcastKill(self.db.global.history[#self.db.global.history]);
		end
		if(timesheet.wipe) then
			MorthalinRaidTools:Print("Raid wiped at " .. encounterName .. " in " .. encounterZone);
		else
			MorthalinRaidTools:Print(encounterName .. " killed at " .. encounterZone .. "(" .. instID .. ")");
		end
	end, 1);
end

function MorthalinRaidTools:ResetHistory()
	self.db.global.history = {};
end

function MorthalinRaidTools:SetZones()
	local rows = self.db.global.history;
	for i=1, #rows do
		if(rows[i].zone == nil) then
			rows[i].zone = self.db.global.encounterList[rows[i].bossName].instance;
		end
	end
end

function MorthalinRaidTools:Reload()
	ReloadUI();
end

function MorthalinRaidTools:Round(num, numDecimalPlaces)
  local mult = 10^(numDecimalPlaces or 0)
  return math.floor(num * mult + 0.5) / mult
end

function MorthalinRaidTools:GetPercentageWithColor(value, maxValue)
	if(maxValue ~= 0) then
		local percentage = self:Round(value * 100 / maxValue, 0);
		if(percentage < 10) then
			return "|cffff0000" .. percentage .. "%";
		elseif(percentage < 20) then
			return "|cffff2a00" .. percentage .. "%";
		elseif(percentage < 30) then
			return "|cffff5500" .. percentage .. "%";
		elseif(percentage < 40) then
			return "|cffff7f00" .. percentage .. "%";
		elseif(percentage < 50) then
			return "|cffffa800" .. percentage .. "%";
		elseif(percentage < 60) then
			return "|cffa8ff00" .. percentage .. "%";
		elseif(percentage < 70) then
			return "|cff7fff00" .. percentage .. "%";
		elseif(percentage < 80) then
			return "|cff55ff00" .. percentage .. "%";
		elseif(percentage < 90) then
			return "|cff2aff00" .. percentage .. "%";
		else
			return "|cff00ff00" .. percentage .. "%";
		end
	else
		return "|cffff0000No kills";
	end
end

function MorthalinRaidTools:GetRatioWithColor(value, maxValue, secondary)
	local percentage = self:Round(value * 100 / maxValue, 0);
	local parentesis = "";
	if(secondary ~= nil and secondary ~= 0) then
		parentesis = "(" .. secondary .. ")";
	end
	if(percentage < 10) then
		return "|cffff0000" .. parentesis .. value .. "/" .. maxValue;
	elseif(percentage < 20) then
		return "|cffff2a00" .. parentesis .. value .. "/" .. maxValue;
	elseif(percentage < 30) then
		return "|cffff5500" .. parentesis .. value .. "/" .. maxValue;
	elseif(percentage < 40) then
		return "|cffff7f00" .. parentesis .. value .. "/" .. maxValue;
	elseif(percentage < 50) then
		return "|cffffa800" .. parentesis .. value .. "/" .. maxValue;
	elseif(percentage < 60) then
		return "|cffa8ff00" .. parentesis .. value .. "/" .. maxValue;
	elseif(percentage < 70) then
		return "|cff7fff00" .. parentesis .. value .. "/" .. maxValue;
	elseif(percentage < 80) then
		return "|cff55ff00" .. parentesis .. value .. "/" .. maxValue;
	elseif(percentage < 90) then
		return "|cff2aff00" .. parentesis .. value .. "/" .. maxValue;
	else
		return "|cff00ff00" .. parentesis .. value .. "/" .. maxValue;
	end
end

function MorthalinRaidTools:ConvertStringIndex(tbl)
	local temp = {};
	for index, data in pairs(tbl) do
		table.insert(temp, data);
		temp[#temp].index = index;
	end
	return temp;
end

function MorthalinRaidTools:SetResetGlobal(reset, instance)
	local dayP, monthP, yearP = reset:match('^(%d%d).(%d%d).(%d%d%d%d)$');
	local resetInt = time{year=yearP, month=monthP, day=dayP, hour=9};
	if(instance == "Onyxia's Lair") then
		MorthalinRaidTools.db.global.resets["Onyx"] = resetInt;
	elseif(instance == "Zul'Gurub" or instance == "Ruins of Ahn'Qiraj") then
		MorthalinRaidTools.db.global.resets["ZG"] = resetInt;
	else
		MorthalinRaidTools.db.global.resets["MC"] = resetInt;
	end
end

function MorthalinRaidTools:SetResetDate(killDate, instance)
	local looping = true;
	local reset;
	local step;
	if(instance == "Onyxia's Lair") then
		reset = MorthalinRaidTools.db.global.resets["Onyx"];
		step = 432000;
	elseif(instance == "Zul'Gurub" or instance == "Ruins of Ahn'Qiraj") then
		reset = MorthalinRaidTools.db.global.resets["ZG"];
		step = 259200;
	else
		reset = MorthalinRaidTools.db.global.resets["MC"];
		step = 604800;
	end
	while looping do
		if(killDate < reset) then
			looping = false;
		else
			reset = reset + step;
		end
	end
	return date("%d.%m.%Y", reset);
end

function MorthalinRaidTools:OnRaidCreated()
	if(self.db.global.checkBenchAuto and self.db.global.acceptBench == false) then
		self:AllowChat();
	end
end

function MorthalinRaidTools:OnRaidLeave()
	if(self.db.global.autoBenchEmpty and #self.db.global.bench.players ~= 0) then
		self:ConstructConfirm("Empty Bench", "Do you want empty bench?", "Empty Bench", 6);
	elseif (self.db.global.askBenchDis and self.db.global.acceptBench) then
		self:ConstructConfirm("Bench invites", "Bench invites are still active. Do you want to disable it?", "Bench invites", 7);
	end
end

function MorthalinRaidTools:FindPlayers(template)
	local temp;
	if(string.len(template) > 1) then
		temp = string.upper(string.utf8sub(template, 1, 1))..string.lower(string.utf8sub(template, 2));
	elseif(string.len(template) == 1) then
		temp = string.upper(string.utf8sub(template, 1, 1));
	end
	local matches = {};
	local numTotalMembers = GetNumGuildMembers();
	for i = 1, numTotalMembers do
		local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i);
		if(temp == string.sub(name, 1, string.len(temp))) then
			table.insert(matches, {text = "|cff" .. MorthalinRaidTools.classColors[class].hex .. string.sub(name, 1, (-1) * (string.len(GetRealmName()) + 2)), name = string.sub(name, 1, (-1) * (string.len(GetRealmName()) + 2)), rank = rankIndex});
		end
	end
	
	local function sortMatches(a, b)
		if(a.rank == b.rank) then
			return a.name < b.name;
		else
			return a.rank < b.rank;
		end
	end
	table.sort(matches, sortMatches);
	
	return matches;
end

function MorthalinRaidTools:GetGuildRanks()
	local ranks = {}
	for i = 1, GuildControlGetNumRanks() do
		table.insert(ranks, GuildControlGetRankName(i));
	end
	return ranks;
end

function MorthalinRaidTools:GetPlayerInfo(template)
	local temp;
	if(string.len(template) > 1) then
		temp = string.upper(string.utf8sub(template, 1, 1))..string.lower(string.utf8sub(template, 2));
	elseif(string.len(template) == 1) then
		temp = string.upper(string.utf8sub(template, 1, 1));
	end
	local numTotalMembers = GetNumGuildMembers();
	for i = 1, numTotalMembers do
		local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i);
		if(temp == string.sub(name, 1, name:find("-") - 1)) then
			return {name = name, class = class, rank = rank, rankIndex = rankIndex, index = i, online = online, officernote = officernote};
		end
	end
	return nil;
end

function MorthalinRaidTools:IsGuildMember(playerName)
	local found = false;
	local i = 1;
	local numTotalMembers = GetNumGuildMembers();
	local realmName = GetRealmName();
	while found == false do
		local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i);
		if(name == playerName .. "-" .. realmName) then
			found = true;
		end
		if(i < numTotalMembers) then
			i = i + 1;
		else
			return found;
		end
	end
	return found;
end

function MorthalinRaidTools:IsMasterLooter(player)
	for i = 1, 40 do
		local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML, combatRole = GetRaidRosterInfo(i);
		if(name ~= nil and Ambiguate(name, "short") == player) then
			return (isML or (UnitIsGroupLeader("raid" .. tostring(i)) and MorthalinRaidTools.db.global.itemRollResetLeader));
		end
	end
	return false;
end

function MorthalinRaidTools:ClearGambleStatistics()
	MorthalinRaidTools.db.global.gambleStatistics = {};
	MorthalinRaidTools:Print("Gamble statistics was cleared.");
end

function MorthalinRaidTools:CheckBuffs()
	MorthalinRaidTools.db.global.members = {};
	MorthalinRaidTools.db.global.debuffs = {};
	MorthalinRaidTools.db.global.buffsSnapshotTime = GetServerTime();
	for i = 1, 40 do
		local name, rank, subgroup, level, class, fileName, zone, online = GetRaidRosterInfo(i);
		if(online) then
			local playerName, _ = UnitName(name);
			if(MorthalinRaidTools.db.global.members[playerName] == nil) then
				MorthalinRaidTools.db.global.members[playerName] = {
					name = playerName,
					class = class,
					color = MorthalinRaidTools.classColors[class].hex,
					group = subgroup,
					buffs = {}
				};
			end
			local buffExist = true;
			local index = 1;
			while(buffExist) do
				local buffName, icon, count, debuffType, duration, expirationTime, source, isStealable, nameplateShowPersonal, spellId = UnitAura(playerName, index);
				if(buffName ~= nil) then
					if(MorthalinRaidTools.db.global.members[playerName].buffs[buffName] == nil) then
						MorthalinRaidTools.db.global.members[playerName].buffs[buffName] = {};
						table.insert(MorthalinRaidTools.db.global.members[playerName].buffs[buffName], {icon = icon, spellId = spellId, count = 1});
					else
						table.insert(MorthalinRaidTools.db.global.members[playerName].buffs[buffName], {icon = icon, spellId = spellId, count = 1});
					end
					index = index + 1;
				else
					buffExist = false;
				end
			end
			MorthalinRaidTools:RemoveExpiredEnchants();
			if(MorthalinRaidTools.db.global.enchants[playerName] ~= nil) then
				MorthalinRaidTools.db.global.members[playerName].buffs[MorthalinRaidTools.db.global.enchants[playerName].name] = {};
				table.insert(MorthalinRaidTools.db.global.members[playerName].buffs[MorthalinRaidTools.db.global.enchants[playerName].name], {icon = MorthalinRaidTools.db.global.enchantList[MorthalinRaidTools.db.global.enchants[playerName].name], spellId = 0, count = 1});
			end
			MorthalinRaidTools.db.global.uniqueBuffs = {};
			for ind, data in pairs(self.db.global.buffs) do
				if(data.spellId ~= nil and data.checkInFight and MorthalinRaidTools.db.global.uniqueBuffs[data.name] == nil) then
					MorthalinRaidTools.db.global.uniqueBuffs[data.name] = {spellId = data.spellId, icon = data.icon};
				end
			end
		end
	end
end

function MorthalinRaidTools:RemoveExpiredEnchants()
	for caster, data in pairs(MorthalinRaidTools.db.global.enchants) do
		if(GetServerTime() > data.expTime) then
			MorthalinRaidTools.db.global.enchants[caster] = nil;
		end
	end
end

function MorthalinRaidTools:CheckRaidSchedule(name, raidDate)
	MorthalinRaidTools:RemoveExpiredSchedule();
	for i = 1, #MorthalinRaidTools.db.global.schedule do
		if(MorthalinRaidTools.db.global.schedule[i].name == name and date("%d.%m.%Y", MorthalinRaidTools.db.global.schedule[i].raidDate) == date("%d.%m.%Y", raidDate)) then
			return true;
		end
	end
	return false;
end

function MorthalinRaidTools:RaidCheck()
	local buffExist = true;
	local found = false;
	local index = 1;
	local missing = {};
	local rest = 0;
	local classes = {};
	
	if(MorthalinRaidTools.db.global.autoReadyCheckOnRC) then
		DoReadyCheck();
	end
	for i = 1, GetNumGroupMembers() do
		local name, rank, subgroup, level, class, fileName, zone, online = GetRaidRosterInfo(i);
		if(online) then
			local playerName, _ = UnitName(name);
			for ind, data in pairs(self.db.global.buffsRaidCheck) do
				found = false;
				buffExist = true;
				rest = 0;
				index = 1;
				while(buffExist) do
					local buffName, icon, count, debuffType, duration, expirationTime, source, isStealable, nameplateShowPersonal, spellId = UnitAura(playerName, index);
					if(buffName ~= nil) then
						if(data.spellNames:find(buffName) ~= nil) then
							rest = expirationTime - GetTime();
							found = true;
							buffExist = false;
						end
						if(data.spellIds:find(spellId) ~= nil) then
							rest = expirationTime - GetTime();
							found = true;
							buffExist = false;
						end
						index = index + 1;
					else
						buffExist = false;
					end
				end
				if(found == false or (found and rest <= data.remain)) then
					if(missing[subgroup] == nil) then
						missing[subgroup] = {};
					end
					if(missing[subgroup][data.name] == nil) then
						missing[subgroup][data.name] = {count = 0, players = {}, caster = data.caster};
					end
					missing[subgroup][data.name].count = missing[subgroup][data.name].count + 1;
					missing[subgroup][data.name].players[playerName] = rest;
				end
			end
			if(classes[class] == nil) then
				classes[class] = {};
			end
			table.insert(classes[class], playerName);
		end
	end
	
	for group, spells in pairs(missing) do
		for spell, players in pairs(spells) do
			if(classes[players.caster] ~= nil or MorthalinRaidTools.db.global.onlyPossibleRC == false) then
				if(players.count > 2) then
					if MorthalinRaidTools.db.global.RCSelf then
						self:Print("Group " .. group .. " needs rebuff: " .. spell);
					else
						SendChatMessage("Group " .. group .. " needs rebuff: " .. spell, "RAID", "Common");
					end
				else
					for player, dur in pairs(players.players) do
						if(dur ~= 0) then
							if MorthalinRaidTools.db.global.RCSelf then
								self:Print(player .. "(" .. group .. ")" .. " needs rebuff: " .. spell .. "(Remaining time: " .. dur .. " s)");
							else
								SendChatMessage(player .. "(" .. group .. ")" .. " needs rebuff: " .. spell .. "(Remaining time: " .. dur .. " s)", "RAID", "Common");
							end
						else
							if MorthalinRaidTools.db.global.RCSelf then
								self:Print(player .. "(" .. group .. ")" .. " needs rebuff: " .. spell);
							else
								SendChatMessage(player .. "(" .. group .. ")"  .. " needs rebuff: " .. spell, "RAID", "Common");
							end
						end
					end
				end
			end
		end
	end
end

function MorthalinRaidTools:RemoveExpiredSchedule()
	local count = #MorthalinRaidTools.db.global.schedule;
	for i = 1, count do
		if(MorthalinRaidTools.db.global.schedule[count - i + 1].raidDate == nil or (MorthalinRaidTools.db.global.schedule[count - i + 1].raidDate < GetServerTime() and date("%d.%m.%Y", MorthalinRaidTools.db.global.schedule[count - i + 1].raidDate) ~= date("%d.%m.%Y", GetServerTime()))) then
			table.remove(MorthalinRaidTools.db.global.schedule, count - i + 1);
		end
	end
end

function MorthalinRaidTools:ReconstructBuffs()
	for i = 1, #self.db.global.history do
		for j = 1, #self.db.global.history[i].members do
			if(self.db.global.history[i].members[j].buffs ~= nil) then
				local buffs = self.db.global.history[i].members[j].buffs;
				for name, icon in pairs(buffs) do
					if(type(icon) == "number") then
						local ic = icon;
						self.db.global.history[i].members[j].buffs[name] = {};
						table.insert(self.db.global.history[i].members[j].buffs[name], {icon = ic, spellId = 0});
					end
				end
			end
		end
	end
end

function MorthalinRaidTools:PlayerRolledForItem(player, roll, range)
	if(range == "(1-" .. MorthalinRaidTools.db.global.itemRollRange .. ")") then
		if(MorthalinRaidTools.db.global.itemRollsCounter[player] == nil) then
			MorthalinRaidTools.db.global.itemRollsCounter[player] = tonumber(roll);
			if(MorthalinRaidTools.ui.itemStatFrame:IsVisible()) then
				MorthalinRaidTools:SortItemStatByRolls();
			end
		end
	end
end

function MorthalinRaidTools:CheckRollPhrase(player, message)
	local res = false;
	local text = message;
	if(MorthalinRaidTools:IsMasterLooter(player)) then
		local startIndex = MorthalinRaidTools.db.global.itemRollResetPhrase:find("<item>");
		if(startIndex ~= nil) then
			if(text:sub(0, startIndex - 1) == MorthalinRaidTools.db.global.itemRollResetPhrase:sub(0, startIndex - 1)) then
				local startI, endI = text:find("|c[^|]+|Hitem[^|]+|[^|]+|h|r");
				if(startI ~= nil and startI == startIndex) then
					text = text:sub(endI + 1, -1);
					if(text == MorthalinRaidTools.db.global.itemRollResetPhrase:sub(startIndex + 6, -1)) then
						res = true;
					end
				end
			end
		end
	end
	return res;
end

function MorthalinRaidTools:CorrectShamanColor()
	for i = 1, #MorthalinRaidTools.db.global.history do
		for j = 1, #MorthalinRaidTools.db.global.history[i].members do
			if(MorthalinRaidTools.db.global.history[i].members[j].class == "Shaman") then
				MorthalinRaidTools.db.global.history[i].members[j].color = MorthalinRaidTools.classColors["Shaman"].hex;
			end
		end
	end
end

function MorthalinRaidTools:AddWipeAttr()
	for i = 1, #MorthalinRaidTools.db.global.history do
		if(MorthalinRaidTools.db.global.history[i].wipe == nil) then
			MorthalinRaidTools.db.global.history[i].wipe = false;
		end
	end
end

function MorthalinRaidTools:Debug(text)
	if(MorthalinRaidTools.db.global.debug) then
		MorthalinRaidTools:Print("|cffffff00DEBUG:|r" .. tostring(text));
	end
end

function MorthalinRaidTools:Test()
	local numTabs = GetNumTalentTabs();
	for t=1, numTabs do
		DEFAULT_CHAT_FRAME:AddMessage(GetTalentTabInfo(t)..":");
		local numTalents = GetNumTalents(t);
		for i=1, numTalents do
			nameTalent, icon, tier, column, currRank, maxRank= GetTalentInfo(t,i);
			DEFAULT_CHAT_FRAME:AddMessage("- "..nameTalent..": "..currRank.."/"..maxRank);
		end
	end
end
