local AceGUI = LibStub("AceGUI-3.0")

local scrollHistory = nil;
local historyBtn = nil;
local uniqueDates = nil;
local historyInstance = nil;
local historyDate = nil;
local historyKill = nil;
local playerToRemove = nil;
local bossToRemove = nil;
local instanceToRemove = nil;
local sortType = 0;
local radioBuffs = nil;
local radioBuffsAll = nil;
local instanceDropDown = nil;
local tooltipHideTimerID = nil;
local buttonHover = nil;
local killHover = nil;

-- History toggle action
-- =========================
function MorthalinRaidTools:ToggleHistory()
	if MorthalinRaidTools.ui.historyFrame:IsVisible() then
		MorthalinRaidTools.ui.historyFrame:Hide();
	else
		MorthalinRaidTools.ui.historyFrame:Show();
		MorthalinRaidTools:UpdateHistory();
	end
end

function MorthalinRaidTools:UpdateHistory()
	if(historyKill ~= nil) then
		MorthalinRaidTools:UpdateHistoryViewPlayers();
		radioBuffsAll.frame:Hide();
	elseif(historyInstance ~= nil) then
		MorthalinRaidTools:UpdateHistoryViewBosses();
		radioBuffsAll.frame:Show();
	elseif(historyDate ~= nil) then
		MorthalinRaidTools:UpdateHistoryViewInstances();
		radioBuffsAll.frame:Hide();
	else
		MorthalinRaidTools:UpdateHistoryView();
		radioBuffsAll.frame:Hide();
	end
end

function MorthalinRaidTools:ConstructHistoryFrame()
	self.ui.historyFrame = AceGUI:Create("Frame");
	self.ui.historyFrame:SetTitle("Kill history");
	self.ui.historyFrame:SetStatusText("");
	self.ui.historyFrame:SetLayout("Flow");
	self.ui.historyFrame:SetWidth(400);
	self.ui.historyFrame:EnableResize(false);
	self.ui.historyFrame.frame:EnableMouse(true);
	self.ui.historyFrame.frame:SetUserPlaced(true);
	self.ui.historyFrame:SetCallback("OnClose", function(widget) self:HideTooltip(); end);
	
	-- TABLE HEADER
	local tableHeader = AceGUI:Create("SimpleGroup");
	tableHeader:SetFullWidth(true);
	tableHeader:SetLayout("Flow");
	self.ui.historyFrame:AddChild(tableHeader);

	historyBtn = AceGUI:Create("InteractiveLabel");
	historyBtn:SetWidth(400);
	historyBtn:SetText("Date");
	historyBtn.highlight:SetColorTexture(0.3, 0.3, 0.3, 0.5);
	historyBtn:SetCallback("OnClick", function() 
		if (sortType == 0) then
			sortType = 1; 
		else
			sortType = 0; 
		end
		self:UpdateHistoryView();
	end);
	tableHeader:AddChild(historyBtn);
	
	--TABLE DATA PART
	scrollcontainer = AceGUI:Create("SimpleGroup");
	scrollcontainer:SetFullWidth(true);
	scrollcontainer:SetHeight(340);
	scrollcontainer:SetLayout("Fill");
	self.ui.historyFrame:AddChild(scrollcontainer);
	scrollcontainer:ClearAllPoints();
	scrollcontainer.frame:SetPoint("TOP", tableHeader.frame, "BOTTOM", 0, -5);
	scrollcontainer.frame:SetPoint("BOTTOM", 0, 20);

	scrollHistory = CreateFrame("ScrollFrame", nil, scrollcontainer.frame, "myScrollFrame");
	HybridScrollFrame_CreateButtons(scrollHistory, "myScrollListItemTemplate");
	HybridScrollFrame_SetDoNotHideScrollBar(scrollHistory, true);
	
	scrollHistory.update = function() 
		MorthalinRaidTools:UpdateHistoryView() 
	end
	
	local radioHeader = AceGUI:Create("SimpleGroup");
	radioHeader:SetFullWidth(true);
	radioHeader:SetLayout("Flow");
	self.ui.historyFrame:AddChild(radioHeader);
	
	instanceDropDown = AceGUI:Create("Dropdown");
	instanceDropDown:SetLabel("Instances");
	instanceDropDown:SetWidth(130);
	instanceDropDown:SetMultiselect(true);
	instanceDropDown:SetCallback("OnValueChanged", function(self, func, key, checked)
		if(historyDate == nil) then
			MorthalinRaidTools:UpdateHistoryView();
		else
			MorthalinRaidTools:UpdateHistoryViewInstances();
		end
	end);
	radioHeader:AddChild(instanceDropDown);
	MorthalinRaidTools:SetInstanceDropDown();
	
	radioBuffs = AceGUI:Create("CheckBox");
	radioBuffs:SetLabel("Show Buffs");
	radioBuffs:SetWidth(100);
	radioBuffs:SetDisabled(true);
	radioBuffs:SetValue(self.db.global.showBuffs);
	radioBuffs:SetCallback("OnValueChanged", function(obj, func, value) 
		self.db.global.showBuffs = value;
		if(value) then
			radioBuffsAll.frame:Show();
		else
			radioBuffsAll.frame:Hide();
		end
	end); 
	radioHeader:AddChild(radioBuffs);
	
	radioBuffsAll = AceGUI:Create("CheckBox");
	radioBuffsAll:SetLabel("Show All Buffs");
	radioBuffsAll:SetWidth(120);
	radioBuffsAll:SetValue(self.db.global.showBuffsAll);
	radioBuffsAll:SetCallback("OnValueChanged", function(obj, func, value) self.db.global.showBuffsAll = value; end); 
	radioHeader:AddChild(radioBuffsAll);
	
	local lable = AceGUI:Create("Label");
	lable:SetFullWidth(375);
	lable:SetText("Click left to open | Click middle to remove | Click right to go back");
	self.ui.historyFrame:AddChild(lable);
	
	lable = AceGUI:Create("Label");
	lable:SetWidth(375);
	lable:SetText("Press ctrl + click left to export | Press ctrl + click right to broadcast");
	self.ui.historyFrame:AddChild(lable);
	
	self.ui.historyFrame:Hide();
end

function MorthalinRaidTools:SetInstanceDropDown()
	instanceDropDown:SetList({});
	local index = 1;
	for zone, shortcut in pairs(self.db.global.instances) do
		instanceDropDown:AddItem(index, shortcut);
		index = index + 1;
	end
	local text;
	for i, widget in instanceDropDown.pullout:IterateItems() do
		if widget.type == "Dropdown-Item-Toggle" then
			widget.value = true;
			widget.check:Show();
			if text then
				text = text..", "..widget:GetText();
			else
				text = widget:GetText();
			end
		end
	end
	instanceDropDown:SetText(text);
end

function MorthalinRaidTools:CheckInstanceValue(value)
	for i, widget in instanceDropDown.pullout:IterateItems() do
		if widget.type == "Dropdown-Item-Toggle" then
			if(widget.value == true and widget:GetText() == value) then
				return true;
			end
		end
	end
	return false;
end

function MorthalinRaidTools:UpdateHistoryView()
	local buttons = HybridScrollFrame_GetButtons(scrollHistory);
	local offset = HybridScrollFrame_GetOffset(scrollHistory);
	
	uniqueDates = self.db.global.history;
	local rows = {};
	local dateTemp = nil;
	for i = 1, #uniqueDates do
		if(MorthalinRaidTools:CheckInstanceValue(self.db.global.instances[uniqueDates[i].zone])) then
			if (date("%H",uniqueDates[i].killDate) == "00" or date("%H",uniqueDates[i].killDate) == "01") then
				dateTemp = date("%d.%m.%Y",(uniqueDates[i].killDate - 7200))
			else
				dateTemp = date("%d.%m.%Y",uniqueDates[i].killDate)
			end
			if (rows[dateTemp] == nil) then
				rows[dateTemp] = {killDate = uniqueDates[i].killDate, instances = self.db.global.instances[uniqueDates[i].zone]};
			else
				if (rows[dateTemp].instances:find(self.db.global.instances[uniqueDates[i].zone]) == nil) then
					rows[dateTemp].instances = rows[dateTemp].instances .. " + " .. self.db.global.instances[uniqueDates[i].zone];
				end
			end
		end
	end
	
	uniqueDates = {};
	for index, value in pairs(rows) do
		table.insert(uniqueDates, value);
	end
	local function sort(a, b)
		if (sortType == 0) then
			return a.killDate > b.killDate;
		else
			return a.killDate < b.killDate;
		end
	end
	table.sort(uniqueDates, sort);
	
	for buttonIndex = 1, #buttons do
		local button = buttons[buttonIndex];
		local itemIndex = buttonIndex + offset;
		
		if (itemIndex <= #uniqueDates) then
			button:SetID(itemIndex);
			button.Boss:SetText(date("%d.%m.%Y",uniqueDates[itemIndex].killDate) .. "    (" .. uniqueDates[itemIndex].instances .. ")");
			button:SetScript('OnEnter', function()
				if(tooltipHideTimerID ~= nil) then
					self:HideTooltip();
					MorthalinRaidTools:CancelTimer(tooltipHideTimerID);
					tooltipHideTimerID = nil;
				end
				self:ShowDatePlayers(date("%d.%m.%Y",uniqueDates[itemIndex].killDate));
			end);
			button:SetScript('OnLeave', function()
				if(MorthalinRaidTools.db.global.autoHideTooltip) then
					tooltipHideTimerID = MorthalinRaidTools:ScheduleTimer(function()
						self:HideTooltip();
						tooltipHideTimerID = nil;
					end, MorthalinRaidTools.db.global.autoHideTooltipTime);
				end
			end);
			button:RegisterForClicks("LeftButtonUp", "RightButtonUp");
			button:SetScript('OnClick', function(self, button, down)
				if(button == "LeftButton") then
					if(IsControlKeyDown()) then
						MorthalinRaidTools:ShowEditWindow(false, 2, date("%d.%m.%Y",uniqueDates[itemIndex].killDate));
					else
						MorthalinRaidTools:SetDatePick(date("%d.%m.%Y",uniqueDates[itemIndex].killDate));
					end
				elseif(button == "RightButton") then
					if(IsControlKeyDown()) then
						MorthalinRaidTools:BroadcastDateFromHistory(date("%d.%m.%Y",uniqueDates[itemIndex].killDate));
					end
				end
			end)
			button:Show();
		else
			button:Hide();
		end
	end

	local buttonHeight = scrollHistory.buttonHeight;
	local totalHeight = #uniqueDates * buttonHeight;
	local shownHeight = #buttons * buttonHeight;

	HybridScrollFrame_Update(scrollHistory, totalHeight, shownHeight);
end

function MorthalinRaidTools:SetDatePick(datePick)
	historyDate = datePick;
	historyBtn:SetText("Instance");
	historyBtn.highlight:SetColorTexture(0, 0, 0, 0);
	historyBtn:SetCallback("OnClick", function() end);
	scrollHistory.update = nil;
	HybridScrollFrame_SetOffset(scrollHistory, 0);
	self:UpdateHistoryViewInstances();
	self:HideTooltip();
	if(tooltipHideTimerID ~= nil) then
		MorthalinRaidTools:CancelTimer(tooltipHideTimerID);
		tooltipHideTimerID = nil;
	end
	scrollHistory.update = function() 
		self:UpdateHistoryViewInstances() 
	end
	historyKill = nil;
end

function MorthalinRaidTools:RemoveDatePick()
	historyDate = nil;
	historyBtn:SetText("Date");
	historyBtn.highlight:SetColorTexture(0.3, 0.3, 0.3, 0.5);
	historyBtn:SetCallback("OnClick", function() 
		if (sortType == 0) then
			sortType = 1; 
		else
			sortType = 0; 
		end
		self:UpdateHistoryView();
	end);
	scrollHistory:SetScript('OnMouseUp',nil);
	self:HideTooltip();
	if(tooltipHideTimerID ~= nil) then
		MorthalinRaidTools:CancelTimer(tooltipHideTimerID);
		tooltipHideTimerID = nil;
	end
	scrollHistory.update = nil;
	HybridScrollFrame_SetOffset(scrollHistory, 0);
	self:UpdateHistoryView();
	scrollHistory.update = function() 
		self:UpdateHistoryView() 
	end
end

function MorthalinRaidTools:UpdateHistoryViewInstances()
	local buttons = HybridScrollFrame_GetButtons(scrollHistory);
	local offset = HybridScrollFrame_GetOffset(scrollHistory);
	local rows = self.db.global.history;
	local instances = {};
	
	for i = 1, #rows do
		if(self.db.global.history[i].instanceID == 1) then
			MorthalinRaidTools:FindRealInstanceID(i);
		end
		if(MorthalinRaidTools:CheckInstanceValue(self.db.global.instances[self.db.global.history[i].zone])) then
			if (date("%H",rows[i].killDate) == "00" or date("%H",rows[i].killDate) == "01") then
				dateTemp = date("%d.%m.%Y",(rows[i].killDate - 7200))
			else
				dateTemp = date("%d.%m.%Y",rows[i].killDate)
			end
			if (dateTemp == historyDate) then
				local found = false;
				for j = 1, #instances do
					if(instances[j].name == rows[i].zone and instances[j].instanceID == rows[i].instanceID) then
						if(instances[j].start > rows[i].killDate) then
							instances[j].start = rows[i].killDate;
						end
						if(instances[j].final < rows[i].killDate) then
							instances[j].final = rows[i].killDate;
						end
						found = true;
					end
				end
				if(found == false) then
					table.insert(instances, {name = rows[i].zone, instanceID = rows[i].instanceID, start = rows[i].killDate, final = rows[i].killDate});
				end
			end
		end
	end
	scrollHistory:SetScript('OnMouseUp', function(obj, button) if(button == "RightButton") then MorthalinRaidTools:RemoveDatePick(); end end);
	
	for buttonIndex = 1, #buttons do
		local button = buttons[buttonIndex];
		local itemIndex = buttonIndex + offset;
		
		if (itemIndex <= #instances ) then
			button:SetID(itemIndex);
			button.Boss:SetText(instances[itemIndex].name .. ": " .. date("%H:%M:%S",instances[itemIndex].start) .. " - " .. date("%H:%M:%S",instances[itemIndex].final));
			button:SetScript('OnEnter', function()
				if(tooltipHideTimerID ~= nil) then
					self:HideTooltip();
					MorthalinRaidTools:CancelTimer(tooltipHideTimerID);
					tooltipHideTimerID = nil;
				end
				self:ShowInstancePlayers(instances[itemIndex])
			end);
			button:SetScript('OnLeave', function()
				if(MorthalinRaidTools.db.global.autoHideTooltip) then
					tooltipHideTimerID = MorthalinRaidTools:ScheduleTimer(function()
						self:HideTooltip();
						tooltipHideTimerID = nil;
					end, MorthalinRaidTools.db.global.autoHideTooltipTime);
				end
			end);
			button:RegisterForClicks("LeftButtonUp", "RightButtonUp", "MiddleButtonUp");
			button:SetScript('OnClick', function(obj, button, down)
				if(button == "RightButton") then
					if(IsControlKeyDown()) then
						MorthalinRaidTools:BroadcastInstanceFromHistory(instances[itemIndex]);
					else
						MorthalinRaidTools:RemoveDatePick();
					end
				elseif(button == "MiddleButton") then
					MorthalinRaidTools:RemoveInstanceFromHistory(instances[itemIndex]);
				else
					if(IsControlKeyDown()) then
						MorthalinRaidTools:ShowEditWindow(false, 3, instances[itemIndex]);
					else
						MorthalinRaidTools:SetInstancePick(instances[itemIndex]);
					end
				end
			end);
			button:Show();
		else
			button:Hide();
		end
	end

	local buttonHeight = scrollHistory.buttonHeight;
	local totalHeight = #instances * buttonHeight;
	local shownHeight = #buttons * buttonHeight;

	HybridScrollFrame_Update(scrollHistory, totalHeight, shownHeight);
end

function MorthalinRaidTools:SetInstancePick(instancePick)
	historyInstance = instancePick;
	historyBtn:SetText("Boss");
	self:HideTooltip();
	if(tooltipHideTimerID ~= nil) then
		MorthalinRaidTools:CancelTimer(tooltipHideTimerID);
		tooltipHideTimerID = nil;
	end
	scrollHistory.update = nil;
	HybridScrollFrame_SetOffset(scrollHistory, 0);
	self:UpdateHistoryViewBosses();
	radioBuffs:SetDisabled(false);
	if(self.db.global.showBuffs) then
		radioBuffsAll.frame:Show();
	end
	instanceDropDown:SetDisabled(true);
	scrollHistory.update = function() 
		self:UpdateHistoryViewBosses() 
	end
end

function MorthalinRaidTools:RemoveInstancePick()
	historyInstance = nil;
	historyBtn:SetText("Instance");
	scrollHistory:SetScript('OnMouseUp',nil);
	scrollHistory.update = nil;
	HybridScrollFrame_SetOffset(scrollHistory, 0);
	self:UpdateHistoryViewInstances();
	radioBuffs:SetDisabled(true);
	radioBuffsAll.frame:Hide();
	self:HideTooltip();
	if(self.ui.iconTable ~= nil and self.ui.iconTable:IsShown()) then
		self.ui.iconTable:Hide();
	end
	if(tooltipHideTimerID ~= nil) then
		MorthalinRaidTools:CancelTimer(tooltipHideTimerID);
		tooltipHideTimerID = nil;
	end
	instanceDropDown:SetDisabled(false);
	scrollHistory.update = function() 
		self:UpdateHistoryViewInstances() 
	end
end

function MorthalinRaidTools:UpdateHistoryViewBosses()
	local buttons = HybridScrollFrame_GetButtons(scrollHistory);
	local offset = HybridScrollFrame_GetOffset(scrollHistory);
	local rows = self.db.global.history;
	local kills = {};
	
	for i = 1, #rows do
		if (date("%H",rows[i].killDate) == "00" or date("%H",rows[i].killDate) == "01") then
			dateTemp = date("%d.%m.%Y",(rows[i].killDate - 7200))
		else
			dateTemp = date("%d.%m.%Y",rows[i].killDate)
		end
		if (dateTemp == historyDate and rows[i].zone == historyInstance.name and rows[i].instanceID == historyInstance.instanceID) then
			table.insert(kills, rows[i]);
		end
	end
	scrollHistory:SetScript('OnMouseUp', function(obj, button) if(button == "RightButton") then MorthalinRaidTools:RemoveInstancePick(); end end);
	
	local function sort(a, b)
		return a.killDate < b.killDate;
	end
	table.sort(kills, sort);
	
	for buttonIndex = 1, #buttons do
		local button = buttons[buttonIndex];
		local itemIndex = buttonIndex + offset;
		
		if (itemIndex <= #kills ) then
			button:SetID(itemIndex);
			local text;
			if(kills[itemIndex].wipe) then
				text = "|cffff0000" .. date("%d.%m.%Y %H:%M:%S",kills[itemIndex].killDate) .. " - " .. kills[itemIndex].bossName;
			else
				text = date("%d.%m.%Y %H:%M:%S",kills[itemIndex].killDate) .. " - " .. kills[itemIndex].bossName .. (MorthalinRaidTools.debug and  "(" .. kills[itemIndex].instanceID .. ")" or "");
			end
			button.Boss:SetText(text);
			button:SetScript('OnEnter', function()
				if(tooltipHideTimerID ~= nil) then
					self:HideTooltip();
					if(self.ui.iconTable ~= nil and self.ui.iconTable:IsShown()) then
						self.ui.iconTable:Hide();
					end
					MorthalinRaidTools:CancelTimer(tooltipHideTimerID);
					tooltipHideTimerID = nil;
				end
				buttonHover = button;
				killHover = kills[itemIndex];
				MorthalinRaidTools:RegisterEvent("MODIFIER_STATE_CHANGED", function(...) MorthalinRaidTools:HistoryModifierChanged(...) end);
				if(IsShiftKeyDown()) then
					GameTooltip:SetOwner(button,"ANCHOR_LEFT");
					local fightTime = killHover.killDate - killHover.pullDate;
					GameTooltip:AddDoubleLine(killHover.bossName, tostring(fightTime) .. " seconds");
					if(killHover.debuffs ~= nil) then
						for name, data in pairs(killHover.debuffs) do
							for i = 1, #data do
								if(MorthalinRaidTools.db.global.timedAuras[name] ~= nil and MorthalinRaidTools.db.global.timedAuras[name].count[i] ~= nil) then
									GameTooltip:AddDoubleLine(name .. " (" .. MorthalinRaidTools.db.global.timedAuras[name].count[i] .. ")", tostring(data[i].seconds) .. "(" .. tostring(fightTime - data[i].seconds) .. ") seconds");
								end
							end
						end
					end
					GameTooltip:Show();
				else
					self:ShowGroups(killHover);
				end
			end);
			button:SetScript('OnLeave', function()
				MorthalinRaidTools:UnregisterEvent("MODIFIER_STATE_CHANGED");
				buttonHover = nil;
				killHover = nil;
				if(MorthalinRaidTools.db.global.autoHideTooltip) then
					if(IsShiftKeyDown()) then
						tooltipHideTimerID = MorthalinRaidTools:ScheduleTimer(function()
							GameTooltip:Hide();
						end, MorthalinRaidTools.db.global.autoHideTooltipTime);
					else
						tooltipHideTimerID = MorthalinRaidTools:ScheduleTimer(function()
							self:HideTooltip();
							if(self.ui.iconTable ~= nil and self.ui.iconTable:IsShown()) then
								self.ui.iconTable:Hide();
							end
							tooltipHideTimerID = nil;
						end, MorthalinRaidTools.db.global.autoHideTooltipTime);
					end
				end
			end);
			button:RegisterForClicks("LeftButtonUp", "RightButtonUp", "MiddleButtonUp");
			button:SetScript('OnClick', function(obj, button, down) 
				if(button == "RightButton") then
					if(IsControlKeyDown()) then
						MorthalinRaidTools:BroadcastKillFromHistory(kills[itemIndex]);
					else
						MorthalinRaidTools:RemoveInstancePick();
					end
				elseif(button == "MiddleButton") then
					MorthalinRaidTools:RemoveBossFromHistory(kills[itemIndex]);
				else
					if(IsControlKeyDown()) then
						MorthalinRaidTools:ShowEditWindow(false, 4, kills[itemIndex]);
					else
						MorthalinRaidTools:SetKillPick(kills[itemIndex]);
					end
				end 
			end);
			button:Show();
		else
			button:Hide();
		end
	end

	local buttonHeight = scrollHistory.buttonHeight;
	local totalHeight = #kills * buttonHeight;
	local shownHeight = #buttons * buttonHeight;

	HybridScrollFrame_Update(scrollHistory, totalHeight, shownHeight);
end

function MorthalinRaidTools:SetKillPick(killPick)
	historyKill = killPick;
	historyBtn:SetText("Players");
	scrollHistory.update = nil;
	HybridScrollFrame_SetOffset(scrollHistory, 0);
	self:UpdateHistoryViewPlayers();
	self:HideTooltip();
	if(self.ui.iconTable ~= nil and self.ui.iconTable:IsShown()) then
		self.ui.iconTable:Hide();
	end
	if(tooltipHideTimerID ~= nil) then
		MorthalinRaidTools:CancelTimer(tooltipHideTimerID);
		tooltipHideTimerID = nil;
	end
	radioBuffs:SetDisabled(true);
	radioBuffsAll.frame:Hide();
	scrollHistory.update = function() 
		self:UpdateHistoryViewPlayers() 
	end
end

function MorthalinRaidTools:UpdateHistoryViewPlayers()
	local buttons = HybridScrollFrame_GetButtons(scrollHistory);
	local offset = HybridScrollFrame_GetOffset(scrollHistory);
	local rows = historyKill.members;
	local function sortByGroup(a, b)
		return a.name < b.name;
	end
	table.sort(rows, sortByGroup);
	scrollHistory:SetScript('OnMouseUp', function(obj, button) if(button == "RightButton") then MorthalinRaidTools:RemoveKillPick(); end end);
	
	for buttonIndex = 1, #buttons do
		local button = buttons[buttonIndex];
		local itemIndex = buttonIndex + offset;
		
		if (itemIndex <= #rows ) then
			button:SetID(itemIndex);
			button.Boss:SetText("|cff" .. rows[itemIndex].color .. rows[itemIndex].name);
			button:SetScript('OnEnter', function()
				if(tooltipHideTimerID ~= nil) then
					if(self.ui.iconToolTip ~= nil) then
						self.ui.iconToolTip:Hide();
					end
					MorthalinRaidTools:CancelTimer(tooltipHideTimerID);
					tooltipHideTimerID = nil;
				end
				MorthalinRaidTools:ShowBuffs(scrollHistory, rows[itemIndex].buffs);
			end);
			button:SetScript('OnLeave', function()
				if(MorthalinRaidTools.db.global.autoHideTooltip) then
					tooltipHideTimerID = MorthalinRaidTools:ScheduleTimer(function()
						if(self.ui.iconToolTip ~= nil) then
							self.ui.iconToolTip:Hide();
						end
						tooltipHideTimerID = nil;
					end, MorthalinRaidTools.db.global.autoHideTooltipTime);
				end
			end);
			button:RegisterForClicks("LeftButtonUp", "RightButtonUp", "MiddleButtonUp");
			button:SetScript('OnClick', function(obj, btn, down)
				if(btn == "RightButton") then 
					MorthalinRaidTools:RemoveKillPick(); 
				elseif(btn == "MiddleButton") then 
					MorthalinRaidTools:RemovePlayerFromKill(itemIndex); 
				end
			end);
			button:Show();
		else
			button:SetScript('OnEnter', nil);
			button:SetScript('OnLeave', nil);
			button:Hide();
		end
	end

	local buttonHeight = scrollHistory.buttonHeight;
	local totalHeight = #rows * buttonHeight;
	local shownHeight = #buttons * buttonHeight;

	HybridScrollFrame_Update(scrollHistory, totalHeight, shownHeight);
end

function MorthalinRaidTools:RemoveKillPick()
	historyKill = nil;
	historyBtn:SetText("Boss");
	scrollHistory.update = nil;
	HybridScrollFrame_SetOffset(scrollHistory, 0);
	if(self.ui.iconToolTip ~= nil and self.ui.iconToolTip:IsShown()) then
		self.ui.iconToolTip:Hide();
	end
	if(tooltipHideTimerID ~= nil) then
		MorthalinRaidTools:CancelTimer(tooltipHideTimerID);
		tooltipHideTimerID = nil;
	end
	self:UpdateHistoryViewBosses();
	radioBuffs:SetDisabled(false);
	if(self.db.global.showBuffs) then
		radioBuffsAll.frame:Show();
	end
	scrollHistory.update = function() 
		self:UpdateHistoryViewBosses() 
	end
end

function MorthalinRaidTools:ShowGroups(kill)
	local history = kill;
	local members = kill.members;
	local function sortByGroup(a, b)
		if(a.group < b.group) then
			return true;
		elseif (a.group > b.group) then
			return false;
		else
			return a.name < b.name;
		end
	end
	table.sort(members, sortByGroup);
	if(radioBuffs:GetValue() == false) then
		if(self.ui.iconTable ~= nil and self.ui.iconTable:IsShown()) then
			self.ui.iconTable:Hide();
		end
		GameTooltip:SetOwner(scrollHistory,"ANCHOR_LEFT");
		GameTooltip:AddLine(history.bossName .. " - " .. date("%d.%m.%Y %H:%M:%S",history.killDate));
		local groups = {};
		for i=0, 4 do
			groups[i] = {players1 = {}, players2 = {}};
		end
		for index, player in pairs(members) do
			if(player.group == 0) then
				table.insert(groups[0].players1, player);
			elseif(math.fmod(player.group, 2) == 0) then
				table.insert(groups[player.group / 2].players2, player);
			else
				table.insert(groups[(player.group + 1) / 2].players1, player);
			end
		end
		
		local groupLoops = 5;
		for i=1, groupLoops do
			if	(i ~= groupLoops) then
				GameTooltip:AddLine(" ");
				GameTooltip:AddDoubleLine("Group " .. ((i * 2) - 1) .. ":", "Group " .. (i * 2) .. ":");
				for j=1, 5 do
					GameTooltip:AddDoubleLine((groups[i].players1[j] == nil and " " or ("|cff" .. groups[i].players1[j].color .. groups[i].players1[j].name)), (groups[i].players2[j] == nil and " " or ("|cff" .. groups[i].players2[j].color .. groups[i].players2[j].name)));
				end
			else
				GameTooltip:AddLine(" ");
				GameTooltip:AddLine("Bench:");
				for index, player in pairs(groups[0].players1) do
					GameTooltip:AddLine("|cff" .. player.color .. player.name);
				end
			end
		end
		GameTooltip:Show();
	else
		GameTooltip:Hide();
		MorthalinRaidTools:ConstructIconTable(scrollHistory, members, kill.pullDate);
	end
end

function MorthalinRaidTools:ShowInstancePlayers(instance)
	local rows = self.db.global.history;
	local kills = {};
	local players = {};
	GameTooltip:SetOwner(scrollHistory,"ANCHOR_LEFT");
	GameTooltip:AddLine("Players");
	for i = 1, #rows do
		if (date("%H",rows[i].killDate) == "00" or date("%H",rows[i].killDate) == "01") then
			dateTemp = date("%d.%m.%Y",(rows[i].killDate - 7200))
		else
			dateTemp = date("%d.%m.%Y",rows[i].killDate)
		end
		if (dateTemp == historyDate and rows[i].zone == instance.name and rows[i].instanceID == instance.instanceID and rows[i].wipe ~= true) then
			table.insert(kills, rows[i]);
		end
	end
	
	for i = 1, #kills do
		local members = kills[i].members;
		for j = 1, #members do
			local playerTemp = {
				name = members[j].name,
				class = members[j].class,
				color = members[j].color,
				group = members[j].group,
				buffs = members[j].buffs
			}
			playerTemp = MorthalinRaidTools:GetMain(playerTemp);
			
			if(players[playerTemp.name] == nil) then
				players[playerTemp.name] = {data = playerTemp, count = 1, bench = 0};
			else
				players[playerTemp.name].count = players[playerTemp.name].count + 1;
			end
			if(members[j].group == 0) then
				players[playerTemp.name].bench = players[playerTemp.name].bench + 1;
			end
		end
	end
	
	players = self:ConvertStringIndex(players);
	local function sortPlayers(a, b)
		return a.index < b.index;
	end
	table.sort(players, sortPlayers);
	
	for i = 1, #players do
		GameTooltip:AddDoubleLine("|cff" .. players[i].data.color .. players[i].data.name, self:GetRatioWithColor(players[i].count, #kills, players[i].bench) .. "  " .. self:GetPercentageWithColor(players[i].count, #kills));
	end
	GameTooltip:Show();
end

function MorthalinRaidTools:ShowDatePlayers(dt)
	local rows = self.db.global.history;
	local kills = {};
	local uniqueKills = {};
	local uniqueKillsCounter = 0;
	local players = {};
	GameTooltip:SetOwner(scrollHistory,"ANCHOR_LEFT");
	GameTooltip:AddLine("Players");
	for i = 1, #rows do
		if (date("%H",rows[i].killDate) == "00" or date("%H",rows[i].killDate) == "01") then
			dateTemp = date("%d.%m.%Y",(rows[i].killDate - 7200))
		else
			dateTemp = date("%d.%m.%Y",rows[i].killDate)
		end
		if (dateTemp == dt and rows[i].wipe ~= true) then
			table.insert(kills, rows[i]);
			if(uniqueKills[rows[i].bossName] == nil) then
				uniqueKills[rows[i].bossName] = 1;
				uniqueKillsCounter = uniqueKillsCounter + 1;
			end
		end
	end
	for i = 1, #kills do
		local members = kills[i].members;
		for j = 1, #members do
			local playerTemp = {
				name = members[j].name,
				class = members[j].class,
				color = members[j].color,
				group = members[j].group,
				buffs = members[j].buffs
			}
			playerTemp = MorthalinRaidTools:GetMain(playerTemp);
			
			if(players[playerTemp.name] == nil) then
				players[playerTemp.name] = {data = playerTemp, count = 1, bench = 0};
			else
				players[playerTemp.name].count = players[playerTemp.name].count + 1;
			end
			if(members[j].group == 0) then
				players[playerTemp.name].bench = players[playerTemp.name].bench + 1;
			end
		end
	end
	
	players = self:ConvertStringIndex(players);
	local function sortPlayers(a, b)
		return a.index < b.index;
	end
	table.sort(players, sortPlayers);
	
	for i = 1, #players do
		GameTooltip:AddDoubleLine("|cff" .. players[i].data.color .. players[i].data.name, self:GetRatioWithColor(players[i].count, uniqueKillsCounter, players[i].bench) .. "  " .. self:GetPercentageWithColor(players[i].count, uniqueKillsCounter));
	end
	GameTooltip:Show();
end

function MorthalinRaidTools:RemovePlayerFromKill(index)
	playerToRemove = index;
	self:ConstructConfirm("Confirmation", "Removing |cff" .. historyKill.members[index].color .. historyKill.members[index].name .. "|r from kill", "Player: " .. historyKill.members[index].name, 1);
end

function MorthalinRaidTools:RemovePlayerConfirmed()
	if(historyKill.members[playerToRemove] ~= nil) then
		table.remove(historyKill.members, playerToRemove);
		MorthalinRaidTools:BroadcastHistoryAdd(historyKill);
		MorthalinRaidTools:UpdateHistoryViewPlayers();
	end
	if(#historyKill.members == 0) then
		local index;
		for i = 1, #self.db.global.history do
			if(self.db.global.history[i] == historyKill) then
				index = i;
			end
		end
		MorthalinRaidTools:BroadcastHistoryRemove(self.db.global.history[index]);
		table.remove(self.db.global.history, index);
		MorthalinRaidTools:RemoveKillPick();
	end
	
	local foundInstance = false;
	local foundDate = false;
	for i = 1, #self.db.global.history do
		local dateTemp;
		if (date("%H",self.db.global.history[i].killDate) == "00" or date("%H",self.db.global.history[i].killDate) == "01") then
			dateTemp = date("%d.%m.%Y",(self.db.global.history[i].killDate - 7200))
		else
			dateTemp = date("%d.%m.%Y",self.db.global.history[i].killDate)
		end
		if (dateTemp == historyDate) then
			if(self.db.global.history[i].zone == historyInstance.name) then
				foundInstance = true;
			end
			foundDate = true;
		end
	end
	if(foundInstance == false) then
		MorthalinRaidTools:RemoveInstancePick();
		if(foundDate == false) then
			MorthalinRaidTools:RemoveDatePick();
		end
	end
end

function MorthalinRaidTools:RemoveBossFromHistory(kill)
	bossToRemove = kill;
	self:ConstructConfirm("Confirmation", "Removing " .. kill.bossName .. " from history", "Boss: " .. kill.bossName, 2);
end

function MorthalinRaidTools:RemoveBossConfirmed()
	local index;
	for i = 1, #self.db.global.history do
		if(self.db.global.history[i] == bossToRemove) then
			index = i;
		end
	end
	MorthalinRaidTools:BroadcastHistoryRemove(self.db.global.history[index]);
	table.remove(self.db.global.history, index);
	MorthalinRaidTools:UpdateHistoryViewBosses();
	
	local foundInstance = false;
	local foundDate = false;
	for i = 1, #self.db.global.history do
		local dateTemp;
		if (date("%H",self.db.global.history[i].killDate) == "00" or date("%H",self.db.global.history[i].killDate) == "01") then
			dateTemp = date("%d.%m.%Y",(self.db.global.history[i].killDate - 7200))
		else
			dateTemp = date("%d.%m.%Y",self.db.global.history[i].killDate)
		end
		if (dateTemp == historyDate) then
			if(self.db.global.history[i].zone == historyInstance.name) then
				foundInstance = true;
			end
			foundDate = true;
		end
	end
	if(foundInstance == false) then
		MorthalinRaidTools:RemoveInstancePick();
		if(foundDate == false) then
			MorthalinRaidTools:RemoveDatePick();
		end
	end
	bossToRemove = nil;
end

function MorthalinRaidTools:RemoveInstanceFromHistory(instance)
	instanceToRemove = instance;
	self:ConstructConfirm("Confirmation", "Removing instance " .. instanceToRemove.name .. " from history", "Instance: " .. instanceToRemove.name, 3);
end

function MorthalinRaidTools:RemoveInstanceConfirmed()
	local count = #self.db.global.history;
	for i = 1, count do
		if (date("%H", self.db.global.history[(count + 1 - i)].killDate) == "00" or date("%H", self.db.global.history[(count + 1 - i)].killDate) == "01") then
			dateTemp = date("%d.%m.%Y",(self.db.global.history[count + 1 - i].killDate - 7200))
		else
			dateTemp = date("%d.%m.%Y",self.db.global.history[count + 1 - i].killDate)
		end
		if (dateTemp == historyDate and self.db.global.history[count + 1 - i].zone == instanceToRemove.name) then
			MorthalinRaidTools:BroadcastHistoryRemove(self.db.global.history[count + 1 - i]);
			table.remove(self.db.global.history, count + 1 - i);
		end
	end
	MorthalinRaidTools:UpdateHistoryViewInstances();
	MorthalinRaidTools:UpdateStatisticsView();
	
	local foundDate = false;
	for i = 1, #self.db.global.history do
		local dateTemp;
		if (date("%H",self.db.global.history[i].killDate) == "00" or date("%H",self.db.global.history[i].killDate) == "01") then
			dateTemp = date("%d.%m.%Y",(self.db.global.history[i].killDate - 7200))
		else
			dateTemp = date("%d.%m.%Y",self.db.global.history[i].killDate)
		end
		if (dateTemp == historyDate) then
			foundDate = true;
		end
	end
	if(foundDate == false) then
		MorthalinRaidTools:RemoveDatePick();
	end
	instanceToRemove = nil;
end

function MorthalinRaidTools:FindRealInstanceID(index)
	if index < #self.db.global.history and self.db.global.history[index].instanceID == 1 then
		for i = index + 1, #self.db.global.history do
			if self.db.global.history[index].resetDate == self.db.global.history[i].resetDate and self.db.global.history[i].instanceID ~= 1 and self.db.global.history[index].zone == self.db.global.history[i].zone then
				if MorthalinRaidTools:CheckSameRaidGroup(index, i) then
					self.db.global.history[index].instanceID = self.db.global.history[i].instanceID;
				end
			end
		end
	end
end

function MorthalinRaidTools:CheckSameRaidGroup(index1, index2)
	local found = false
	local i, j = 1, 1;
	while found == false do
		if(self.db.global.history[index2].members[j] == nil) then
			j = 1;
			i = i + 1;
		end
		if self.db.global.history[index1].members[i] == nil then
			return found;
		end
		if self.db.global.history[index1].members[i].name == self.db.global.history[index2].members[j].name then
			found = true;
		else
			j = j + 1;
		end
	end
	return found;
end

function MorthalinRaidTools:ShowBuffs(parent, buffs)
	if(buffs ~= nil) then
		MorthalinRaidTools:ConstructIconToolTip(parent, buffs);
	end
end

function MorthalinRaidTools:HistoryModifierChanged(...)
	local modifier = select(2, ...);
	local num = select(3, ...);
	if(modifier == "LSHIFT" or modifier == "RSHIFT") then
		if(tooltipHideTimerID ~= nil) then
			self:HideTooltip();
			if(self.ui.iconTable ~= nil and self.ui.iconTable:IsShown()) then
				self.ui.iconTable:Hide();
			end
			MorthalinRaidTools:CancelTimer(tooltipHideTimerID);
			tooltipHideTimerID = nil;
		end
		if(num == 0) then
			GameTooltip:Hide();
			self:ShowGroups(killHover);
		else
			if(self.ui.iconTable ~= nil and self.ui.iconTable:IsShown()) then
				self.ui.iconTable:Hide();
			end
			GameTooltip:SetOwner(buttonHover,"ANCHOR_LEFT");
			local fightTime = killHover.killDate - killHover.pullDate;
			GameTooltip:AddDoubleLine(killHover.bossName, tostring(fightTime) .. " seconds");
			if(killHover.debuffs ~= nil) then
				for name, data in pairs(killHover.debuffs) do
					for i = 1, #data do
						if(MorthalinRaidTools.db.global.timedAuras[name] ~= nil and MorthalinRaidTools.db.global.timedAuras[name].count[i] ~= nil) then
							GameTooltip:AddDoubleLine(name .. " (" .. MorthalinRaidTools.db.global.timedAuras[name].count[i] .. ")", tostring(data[i].seconds) .. "(" .. tostring(fightTime - data[i].seconds) .. ") seconds");
						end
					end
				end
			end
			GameTooltip:Show();
		end
	end
end

function MorthalinRaidTools:GetMain(data)
	local res = data;
	local isAlt = false;
	if(MorthalinRaidTools.db.global.showMainsHistory) then
		for i=1, GetNumGuildMembers() do
			local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i);
			if(string.sub(name, 1, name:find("-") - 1) == res.name) then
				if(string.sub(officernote, 1, 4) == "Alt:") then
					res.name = string.sub(officernote, 5, -1);
					res.class = class;
					res.color = MorthalinRaidTools.classColors[class].hex;
					isAlt = true;
					break;
				end
			end
		end
		if(isAlt) then
			for i=1, GetNumGuildMembers() do
				local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i);
				if(string.sub(name, 1, name:find("-") - 1) == res.name) then
					res.class = class;
					res.color = MorthalinRaidTools.classColors[class].hex;
					break;
				end
			end
		end
	end
	return res;
end
