local AceGUI = LibStub("AceGUI-3.0")
local LibDataBroker = LibStub:GetLibrary("LibDataBroker-1.1", true)

local killControl = nil;
local btnAlts = nil;
local tooltipHideTimerID = nil;
local btnBuffSnapshot = nil;

-- MiniMap Icon Definition
-- =========================
function MorthalinRaidTools:ConstructMiniMapIcon() 
	LibStub("LibDBIcon-1.0"):Register("MorthalinRaidTools", LibStub("LibDataBroker-1.1"):NewDataObject("MorthalinRaidTools",
	{
		type = "data source",
		text = addonName,
		icon = "Interface\\AddOns\\MorthalinRaidTools\\media\\icon",
		OnClick = function(...) self:ToggleMenu() end,
		OnTooltipShow = function(tooltip)
			tooltip:AddLine("Morthalin Raid Tools",1,1,1);
			tooltip:Show();
		end
	}), self.db.char.minimap);
	if LibDataBroker then
		LibDataBroker:NewDataObject("Morthalin Raid Tools", {
			type = "launcher",
			icon = "Interface\\AddOns\\MorthalinRaidTools\\media\\icon",
			OnClick = function(...) self:ToggleMenu() end,
		})
	end
end

function MorthalinRaidTools:ToggleMinimap()
	local icon = LibStub("LibDBIcon-1.0")
	if(self.db.char.minimap.hide) then
		self.db.char.minimap.hide = false;
		icon:Show("MorthalinRaidTools");
	else
		self.db.char.minimap.hide = true;
		icon:Hide("MorthalinRaidTools");
	end
end

-- Menu toggle action
-- =========================
function MorthalinRaidTools:ToggleMenu()
	if MorthalinRaidTools.ui.menuFrame:IsVisible() then
		MorthalinRaidTools.ui.menuFrame:Hide();
		if(MorthalinRaidTools.ui.historyFrame:IsVisible()) then
			MorthalinRaidTools.ui.historyFrame:Hide();
		end
		if MorthalinRaidTools.ui.benchFrame:IsVisible() then
			MorthalinRaidTools.ui.benchFrame:Hide();
		end
	elseif(InCombatLockdown() == false) then
		MorthalinRaidTools.ui.menuFrame:Show();
	end
	
	if(CanEditOfficerNote() or MorthalinRaidTools.debug) then
		btnAlts:SetDisabled(false);
	else
		btnAlts:SetDisabled(true);
	end
end

function MorthalinRaidTools:ConstructUI()
	MorthalinRaidTools.ui = {};
	MorthalinRaidTools:ConstructGamble();
	MorthalinRaidTools:ConstructRaidCheckSettings();
	MorthalinRaidTools:ConstructHistorySettings();
	MorthalinRaidTools:ConstructHistoryFrame();
	MorthalinRaidTools:ConstructBenchFrame();
	MorthalinRaidTools:ConstructStatisticsFrame();
	MorthalinRaidTools:ConstructEditFrame();
	MorthalinRaidTools:ConstructAltsSettings();
	MorthalinRaidTools:ConstructSchedulerFrame();
	MorthalinRaidTools:ConstructItemStatFrame();
	MorthalinRaidTools:ConstructMenuFrame();
end

function MorthalinRaidTools:ConstructMenuFrame()
	self.ui.menuFrame = AceGUI:Create("Frame");
	self.ui.menuFrame:SetTitle("Menu");
	self.ui.menuFrame:SetStatusText("Main menu v"  .. self.db.global.version);
	self.ui.menuFrame:SetCallback("OnClose", function(widget)
		if(MorthalinRaidTools.ui.historyFrame:IsVisible()) then 
			MorthalinRaidTools.ui.historyFrame:Hide(); 
		end
		if(MorthalinRaidTools.ui.benchFrame:IsVisible()) then 
			MorthalinRaidTools.ui.benchFrame:Hide(); 
		end
		if(MorthalinRaidTools.ui.statisticsFrame:IsVisible()) then 
			MorthalinRaidTools.ui.statisticsFrame:Hide(); 
		end
		if(tooltipHideTimerID ~= nil) then
			MorthalinRaidTools:CancelTimer(tooltipHideTimerID);
			tooltipHideTimerID = nil;
		end
		if(self.ui.iconTable ~= nil and self.ui.iconTable:IsShown()) then
			self.ui.iconTable:Hide();
		end
		MorthalinRaidTools:HideHistorySettings();
	end);
	self.ui.menuFrame:SetLayout("Flow");
	self.ui.menuFrame:SetWidth(400);
	if(self.db.global.debug) then
		self.ui.menuFrame:SetHeight(235);
	else
		self.ui.menuFrame:SetHeight(210);
	end
	self.ui.menuFrame:EnableResize(false);
	self.ui.menuFrame.frame:EnableMouse(true);
	self.ui.menuFrame.frame:SetUserPlaced(true);
	_G["MyGlobalFrameName"] = self.ui.menuFrame.frame;
    tinsert(UISpecialFrames, "MyGlobalFrameName");
	
	local btnBench = AceGUI:Create("Button");
	btnBench:SetText("Bench");
	btnBench:SetWidth(180);
	btnBench:SetCallback("OnClick", function() self:ToggleBench(); end);
	btnBench:SetCallback("OnEnter", function() self:ShowBench(btnBench.frame) end);
	btnBench:SetCallback("OnLeave", function() self:HideTooltip() end);
	self.ui.menuFrame:AddChild(btnBench);
	
	local btnHistory = AceGUI:Create("Button");
	btnHistory:SetText("History");
	btnHistory:SetWidth(180);
	btnHistory:SetCallback("OnClick", function() self:ToggleHistory(); end);
	self.ui.menuFrame:AddChild(btnHistory);
	
	local btnBench = AceGUI:Create("Button");
	btnBench:SetText("Scheduler");
	btnBench:SetWidth(180);
	btnBench:SetCallback("OnClick", function() self:ToggleScheduler(); end);
	self.ui.menuFrame:AddChild(btnBench);
	
	local btnStatistics = AceGUI:Create("Button");
	btnStatistics:SetText("Statistics");
	btnStatistics:SetWidth(180);
	btnStatistics:SetCallback("OnClick", function() self:ToggleStatistics(); end);
	self.ui.menuFrame:AddChild(btnStatistics);
	
	btnBuffSnapshot = AceGUI:Create("Button");
	btnBuffSnapshot:SetText("Buffs Snapshot");
	btnBuffSnapshot:SetWidth(180);
	btnBuffSnapshot:SetCallback("OnClick", function()
		if(tooltipHideTimerID ~= nil) then
			MorthalinRaidTools:CancelTimer(tooltipHideTimerID);
			tooltipHideTimerID = nil;
		end
		MorthalinRaidTools:CheckBuffs();
		local membersIndexed = {};
		for name, data in pairs(MorthalinRaidTools.db.global.members) do
			table.insert(membersIndexed, data);
		end
		if(#membersIndexed > 1) then
			MorthalinRaidTools:ConstructIconTable(btnBuffSnapshot.frame, membersIndexed, MorthalinRaidTools.db.global.buffsSnapshotTime);
		end
	end);
	btnBuffSnapshot:SetCallback("OnEnter", function()
		MorthalinRaidTools:RegisterEvent("MODIFIER_STATE_CHANGED", function(...) MorthalinRaidTools:SnapshotModifierChanged(...) end);
		if(tooltipHideTimerID ~= nil) then
			MorthalinRaidTools:CancelTimer(tooltipHideTimerID);
			tooltipHideTimerID = nil;
		end
		if(IsShiftKeyDown()) then
			GameTooltip:SetOwner(btnBuffSnapshot.frame,"ANCHOR_LEFT");
			GameTooltip:AddDoubleLine("Debuff times");
			for name, data in pairs(MorthalinRaidTools.db.global.debuffs) do
				for i = 1, #data do
					if(MorthalinRaidTools.db.global.timedAuras[name] ~= nil) then
						GameTooltip:AddDoubleLine(name .. " (" .. MorthalinRaidTools.db.global.timedAuras[name].count[i] .. ")", tostring(data[i].seconds) .. " seconds");
					end
				end
			end
			GameTooltip:Show();
		else
			local membersIndexed = {};
			for name, data in pairs(MorthalinRaidTools.db.global.members) do
				table.insert(membersIndexed, data);
			end
			if(#membersIndexed > 1) then
				MorthalinRaidTools:ConstructIconTable(btnBuffSnapshot.frame, membersIndexed, MorthalinRaidTools.db.global.buffsSnapshotTime);
			end
		end
	end);
	btnBuffSnapshot:SetCallback("OnLeave", function()
		MorthalinRaidTools:UnregisterEvent("MODIFIER_STATE_CHANGED");
		if(IsShiftKeyDown()) then
			if(MorthalinRaidTools.db.global.autoHideTooltipSnapshot) then
				tooltipHideTimerID = MorthalinRaidTools:ScheduleTimer(function()
					GameTooltip:Hide();
				end, MorthalinRaidTools.db.global.autoHideTooltipSnapshotTime);
			end
		else
			if(MorthalinRaidTools.db.global.autoHideTooltipSnapshot) then
				tooltipHideTimerID = MorthalinRaidTools:ScheduleTimer(function()
					if(self.ui.iconTable ~= nil and self.ui.iconTable:IsShown()) then
						self.ui.iconTable:Hide();
					end
				end, MorthalinRaidTools.db.global.autoHideTooltipSnapshotTime);
			end
		end
	end);
	self.ui.menuFrame:AddChild(btnBuffSnapshot);
	
	local btnItemStats = AceGUI:Create("Button");
	btnItemStats:SetText("Item Statistics");
	btnItemStats:SetWidth(180);
	btnItemStats:SetCallback("OnClick", function() MorthalinRaidTools:ToggleItemStat(); end);
	self.ui.menuFrame:AddChild(btnItemStats);
	
	local btnRaidCheck = AceGUI:Create("Button");
	btnRaidCheck:SetText("Raid Check");
	btnRaidCheck:SetWidth(180);
	btnRaidCheck:SetCallback("OnClick", function() MorthalinRaidTools:RaidCheck(); end);
	self.ui.menuFrame:AddChild(btnRaidCheck);
	
	btnAlts = AceGUI:Create("Button");
	btnAlts:SetText("Alts");
	btnAlts:SetWidth(180);
	btnAlts:SetDisabled(true);
	btnAlts:SetCallback("OnClick", function() MorthalinRaidTools:ShowAltSettings(); end);
	self.ui.menuFrame:AddChild(btnAlts);
	
	local btnGamble = AceGUI:Create("Button");
	btnGamble:SetText("Gamble");
	btnGamble:SetWidth(180);
	btnGamble:SetCallback("OnClick", function() MorthalinRaidTools:ShowGamble(); end);
	self.ui.menuFrame:AddChild(btnGamble);
	
	local btnOptions = AceGUI:Create("Button");
	btnOptions:SetText("Options");
	btnOptions:SetWidth(180);
	btnOptions:SetCallback("OnClick", function()
		InterfaceOptionsFrame_OpenToCategory("Morthalin Raid Tools");
		InterfaceOptionsFrame_OpenToCategory("Morthalin Raid Tools");--Double tap if not working on first try.
		MorthalinRaidTools:ToggleMenu(); end);
	self.ui.menuFrame:AddChild(btnOptions);
	
	if(self.db.global.debug) then
		local testHeader = AceGUI:Create("SimpleGroup");
		testHeader:SetFullWidth(true);
		testHeader:SetLayout("Flow");
		self.ui.menuFrame:AddChild(testHeader);
		testHeader:ClearAllPoints();
		testHeader:SetPoint("BOTTOMLEFT", self.ui.menuFrame.frame, 0, 5);
		
		local btnTest = AceGUI:Create("Button");
		btnTest:SetText("Test");
		btnTest:SetWidth(120);
		btnTest:SetCallback("OnClick", function() self:Test(); end)
		testHeader:AddChild(btnTest);
		
		local btnReload = AceGUI:Create("Button");
		btnReload:SetText("Reload");
		btnReload:SetWidth(120);
		btnReload:SetCallback("OnClick", function() self:Reload(); end)
		testHeader:AddChild(btnReload);
	end
	self.ui.menuFrame:Hide();
end

function MorthalinRaidTools:HideTooltip()
	if(GameTooltip:IsVisible()) then
		GameTooltip:Hide();
	end
end

function MorthalinRaidTools:ConstructConfirm(title, text, status, typeOfConfirm)
	if(self.ui.confirmFrame ~= nil and self.ui.confirmFrame:IsVisible()) then
		self.ui.confirmFrame:Hide();
	end
	self.ui.confirmFrame = AceGUI:Create("Frame");
	self.ui.confirmFrame:SetTitle(title);
	self.ui.confirmFrame:SetStatusText(status);
	self.ui.confirmFrame:SetCallback("OnClose", function(widget) AceGUI:Release(widget) end);
	self.ui.confirmFrame:SetLayout("Flow");
	self.ui.confirmFrame:SetWidth(350);
	self.ui.confirmFrame:SetHeight(115);
	self.ui.confirmFrame:EnableResize(false);
	
	local label = AceGUI:Create("Label");
	label:SetText(text);
	label:SetFullWidth(true);
	self.ui.confirmFrame:AddChild(label);
	
	local btnYes = AceGUI:Create("Button");
	btnYes:SetText("Yes");
	btnYes:SetWidth(150);
	btnYes:SetCallback("OnClick", function() 
		if(typeOfConfirm == 1) then
			self:RemovePlayerConfirmed();
			self.ui.confirmFrame:Hide();
		elseif(typeOfConfirm == 2) then
			self:RemoveBossConfirmed();
			self.ui.confirmFrame:Hide();
		elseif(typeOfConfirm == 3) then
			self:RemoveInstanceConfirmed();
			self.ui.confirmFrame:Hide();
		elseif(typeOfConfirm == 4) then
			self:AllowChat();
			self.ui.confirmFrame:Hide();
		elseif(typeOfConfirm == 5) then
			self:AllowKills();
			self.ui.confirmFrame:Hide();
		elseif(typeOfConfirm == 6) then
			self:ResetList();
			self.ui.confirmFrame:Hide();
			if(self.db.global.askBenchDis and self.db.global.acceptBench) then
				self:ConstructConfirm("Bench invites", "Bench invites are still active. Do you want to disable it?", "Bench invites", 7);
			end
		elseif(typeOfConfirm == 7) then
			self:AllowChat();
			self.ui.confirmFrame:Hide();
		elseif(typeOfConfirm == 8) then
			self:DeletePlayerComfirmed();
			self.ui.confirmFrame:Hide();
		elseif(typeOfConfirm == 9) then
			self:AddAltContinue();
			self.ui.confirmFrame:Hide();
		elseif(typeOfConfirm == 10) then
			self:ChangeNewMainToHistory();
			self.ui.confirmFrame:Hide();
		end
	end)
	self.ui.confirmFrame:AddChild(btnYes);
	
	local btnNo = AceGUI:Create("Button");
	btnNo:SetText("No");
	btnNo:SetWidth(150);
	btnNo:SetCallback("OnClick", function() self.ui.confirmFrame:Hide(); end)
	self.ui.confirmFrame:AddChild(btnNo);
end

function MorthalinRaidTools:SnapshotModifierChanged(...)
	local modifier = select(2, ...);
	local num = select(3, ...);
	if(modifier == "LSHIFT" or modifier == "RSHIFT") then
		if(tooltipHideTimerID ~= nil) then
			MorthalinRaidTools:CancelTimer(tooltipHideTimerID);
			tooltipHideTimerID = nil;
		end
		if(num == 0) then
			GameTooltip:Hide();
			local membersIndexed = {};
			for name, data in pairs(MorthalinRaidTools.db.global.members) do
				table.insert(membersIndexed, data);
			end
			if(#membersIndexed > 1) then
				MorthalinRaidTools:ConstructIconTable(btnBuffSnapshot.frame, membersIndexed, MorthalinRaidTools.db.global.buffsSnapshotTime);
			end
		else
			if(self.ui.iconTable ~= nil and self.ui.iconTable:IsShown()) then
				self.ui.iconTable:Hide();
			end
			GameTooltip:SetOwner(btnBuffSnapshot.frame,"ANCHOR_LEFT");
			GameTooltip:AddDoubleLine("Debuff times");
			for name, data in pairs(MorthalinRaidTools.db.global.debuffs) do
				for i = 1, #data do
					GameTooltip:AddDoubleLine(name .. " (" .. MorthalinRaidTools.db.global.timedAuras[name].count[i] .. ")", tostring(data[i].seconds) .. " seconds");
				end
			end
			GameTooltip:Show();
		end
	end
end
