local AceGUI = LibStub("AceGUI-3.0")

local buffsPick, enabledCHB, buffName, remainTime, spellNames, spellIdEdit, classCasterPick, classPick, includePick, excludePick;
local classes;

function MorthalinRaidTools:ConstructRaidCheckSettings()
	self.ui.raidCheckSetting = AceGUI:Create("Frame");
	self.ui.raidCheckSetting:SetTitle("Raid Check settings");
	self.ui.raidCheckSetting:SetStatusText("");
	self.ui.raidCheckSetting:SetLayout("Flow");
	self.ui.raidCheckSetting:SetWidth(445);
	self.ui.raidCheckSetting:SetHeight(450);
	self.ui.raidCheckSetting:EnableResize(false);
	self.ui.raidCheckSetting.frame:EnableMouse(true);
	self.ui.raidCheckSetting.frame:SetUserPlaced(true);
	
	buffsPick = AceGUI:Create("Dropdown");
	buffsPick:SetLabel("Buffs");
	buffsPick:SetWidth(360);
	buffsPick:SetCallback("OnValueChanged", function(obj, func, value) MorthalinRaidTools:BuffPickedRC() end);
	self.ui.raidCheckSetting:AddChild(buffsPick);
	
	local addBuff = AceGUI:Create("Icon");
	addBuff:SetImage("Interface\\Icons\\spell_chargepositive");
	addBuff:SetWidth(20);
	addBuff:SetHeight(20);
	addBuff:SetImageSize(17,17);
	addBuff:SetCallback("OnClick", function() MorthalinRaidTools:AddBuffRC() end);
	self.ui.raidCheckSetting:AddChild(addBuff);
	
	local removeBuff = AceGUI:Create("Icon");
	removeBuff:SetImage("Interface\\Icons\\spell_chargenegative");
	removeBuff:SetWidth(20);
	removeBuff:SetHeight(20);
	removeBuff:SetImageSize(15,15);
	removeBuff:SetCallback("OnClick", function() MorthalinRaidTools:RemoveBuffRC(buffsPick:GetValue()); end);
	self.ui.raidCheckSetting:AddChild(removeBuff);
	
	enabledCHB = AceGUI:Create("CheckBox");
	enabledCHB:SetLabel("Enabled");
	enabledCHB:SetWidth(200);
	enabledCHB:SetCallback("OnValueChanged", function(obj, func, value) --[[MorthalinRaidTools:SetEnabledRC(value);--]] end); 
	self.ui.raidCheckSetting:AddChild(enabledCHB);
	
	buffName = AceGUI:Create("EditBox");
	buffName:SetLabel("Buff Name");
	buffName:SetWidth(250);
	buffName:SetCallback("OnEnterPressed", function(widget, event, text) MorthalinRaidTools:SetBuffNameRC(text) end);
	self.ui.raidCheckSetting:AddChild(buffName);
	
	remainTime = AceGUI:Create("EditBox");
	remainTime:SetLabel("Warning time(seconds)");
	remainTime:SetWidth(150);
	remainTime:SetCallback("OnEnterPressed", function(widget, event, text) MorthalinRaidTools:SetRemTimeRC(text) end);
	self.ui.raidCheckSetting:AddChild(remainTime);
	
	spellNames = AceGUI:Create("EditBox");
	spellNames:SetLabel("Spell Names (separator\",\")");
	spellNames:SetWidth(400);
	spellNames:SetCallback("OnEnterPressed", function(widget, event, text) MorthalinRaidTools:SetSpellNamesRC(text) end);
	self.ui.raidCheckSetting:AddChild(spellNames);
	
	spellIdEdit = AceGUI:Create("EditBox");
	spellIdEdit:SetLabel("Spell IDs (separator\",\")");
	spellIdEdit:SetWidth(400);
	spellIdEdit:SetCallback("OnEnterPressed", function(widget, event, text) MorthalinRaidTools:SetSpellIdsRC(text) end);
	self.ui.raidCheckSetting:AddChild(spellIdEdit);
	
	classCasterPick = AceGUI:Create("Dropdown");
	classCasterPick:SetLabel("Caster Class");
	classCasterPick:SetWidth(400);
	classCasterPick:SetCallback("OnValueChanged", function(self, func, key, checked) MorthalinRaidTools:GetPickedCasterRC(); end);
	self.ui.raidCheckSetting:AddChild(classCasterPick);
	
	classPick = AceGUI:Create("Dropdown");
	classPick:SetLabel("Classes");
	classPick:SetWidth(400);
	classPick:SetMultiselect(true);
	classPick:SetCallback("OnValueChanged", function(self, func, key, checked) MorthalinRaidTools:GetPickedClassesRC(); end);
	self.ui.raidCheckSetting:AddChild(classPick);
	
	includePick = AceGUI:Create("Dropdown");
	includePick:SetLabel("Include");
	includePick:SetWidth(160);
	includePick:SetCallback("OnValueChanged", function() MorthalinRaidTools:IncludePickedRC(); end);
	self.ui.raidCheckSetting:AddChild(includePick);
	
	local addInclude = AceGUI:Create("Icon");
	addInclude:SetImage("Interface\\Icons\\spell_chargepositive");
	addInclude:SetWidth(20);
	addInclude:SetHeight(20);
	addInclude:SetImageSize(15,15);
	addInclude:SetCallback("OnClick", function() MorthalinRaidTools:AddIncludeRC(); end);
	self.ui.raidCheckSetting:AddChild(addInclude);
	
	local removeInclude = AceGUI:Create("Icon");
	removeInclude:SetImage("Interface\\Icons\\spell_chargenegative");
	removeInclude:SetWidth(20);
	removeInclude:SetHeight(20);
	removeInclude:SetImageSize(15,15);
	removeInclude:SetCallback("OnClick", function() MorthalinRaidTools:RemoveIncludeRC(); end);
	self.ui.raidCheckSetting:AddChild(removeInclude);
	
	excludePick = AceGUI:Create("Dropdown");
	excludePick:SetLabel("Exclude");
	excludePick:SetWidth(160);
	excludePick:SetCallback("OnValueChanged", function() MorthalinRaidTools:ExcludePickedRC(); end);
	self.ui.raidCheckSetting:AddChild(excludePick);
	
	local addExclude = AceGUI:Create("Icon");
	addExclude:SetImage("Interface\\Icons\\spell_chargepositive");
	addExclude:SetWidth(20);
	addExclude:SetHeight(20);
	addExclude:SetImageSize(15,15);
	addExclude:SetCallback("OnClick", function() MorthalinRaidTools:AddExcludeRC(); end);
	self.ui.raidCheckSetting:AddChild(addExclude);
	
	local removeExclude = AceGUI:Create("Icon");
	removeExclude:SetImage("Interface\\Icons\\spell_chargenegative");
	removeExclude:SetWidth(20);
	removeExclude:SetHeight(20);
	removeExclude:SetImageSize(15,15);
	removeExclude:SetCallback("OnClick", function() MorthalinRaidTools:RemoveExcludeRC(); end);
	self.ui.raidCheckSetting:AddChild(removeExclude);
	
	includeName = AceGUI:Create("EditBox");
	includeName:SetLabel("");
	includeName:SetWidth(200);
	includeName:SetCallback("OnEnterPressed", function(widget, event, text) MorthalinRaidTools:SetIncludeNameRC(text); end);
	self.ui.raidCheckSetting:AddChild(includeName);
	
	excludeName = AceGUI:Create("EditBox");
	excludeName:SetLabel("");
	excludeName:SetWidth(200);
	excludeName:SetCallback("OnEnterPressed", function(widget, event, text) MorthalinRaidTools:SetExcludeNameRC(text); end);
	self.ui.raidCheckSetting:AddChild(excludeName);
	
	MorthalinRaidTools:LoadBuffsRC();
	MorthalinRaidTools:SetClassesPickRC();
	MorthalinRaidTools:HideRaidCheckSettings();
end

function MorthalinRaidTools:ShowRaidCheckSettings()
	if(self.ui.raidCheckSetting:IsVisible() == false) then
		self.ui.raidCheckSetting:Show();
	end
end

function MorthalinRaidTools:HideRaidCheckSettings()
	if(self.ui.raidCheckSetting:IsVisible()) then
		self.ui.raidCheckSetting:Hide();
	end
end

function MorthalinRaidTools:SetClassesPickRC()
	classPick:SetList({});
	classPick:AddItem(1, "|cffFF7D0ADruid");
	classPick:AddItem(2, "|cffABD473Hunter");
	classPick:AddItem(3, "|cff40C7EBMage");
	classPick:AddItem(4, "|cffFFFFFFPriest");
	classPick:AddItem(5, "|cffFFF569Rogue");
	classPick:AddItem(6, "|cffF58CBAShaman");
	classPick:AddItem(7, "|cffF58CBAPaladin");
	classPick:AddItem(8, "|cff8787EDWarlock");
	classPick:AddItem(9, "|cffC79C6EWarrior");
	classCasterPick:SetList({});
	classCasterPick:AddItem(1, "|cffFF7D0ADruid");
	classCasterPick:AddItem(2, "|cffABD473Hunter");
	classCasterPick:AddItem(3, "|cff40C7EBMage");
	classCasterPick:AddItem(4, "|cffFFFFFFPriest");
	classCasterPick:AddItem(5, "|cffFFF569Rogue");
	classCasterPick:AddItem(6, "|cffF58CBAShaman");
	classCasterPick:AddItem(7, "|cffF58CBAPaladin");
	classCasterPick:AddItem(8, "|cff8787EDWarlock");
	classCasterPick:AddItem(9, "|cffC79C6EWarrior");
	classCasterPick:AddItem(10, "");
	classes = {};
	classes["Druid"] = 1;
	classes["Hunter"] = 2;
	classes["Mage"] = 3;
	classes["Priest"] = 4;
	classes["Rogue"] = 5;
	classes["Shaman"] = 6;
	classes["Paladin"] = 7;
	classes["Warlock"] = 8;
	classes["Warrior"] = 9;
end

function MorthalinRaidTools:LoadBuffsRC()
	local function sortBuffs(a, b)
		if(a.spellType == b.spellType) then
			return a.name < b.name;
		else
			return (a.spellType or 5) < (b.spellType or 5);
		end
	end
	table.sort(self.db.global.buffsRaidCheck, sortBuffs);
	local index = 1;
	buffsPick:SetValue(nil);
	buffsPick:SetList({});
	for ind, data in pairs(self.db.global.buffsRaidCheck) do
		buffsPick:AddItem(index, data.name);
		index = index + 1;
	end
	MorthalinRaidTools:BuffPickedRC();
end

function MorthalinRaidTools:BuffPickedRC()
	if(buffsPick:GetValue() ~= nil) then
		buffName:SetText(self.db.global.buffsRaidCheck[buffsPick:GetValue()].name);
		
		if(self.db.global.buffsRaidCheck[buffsPick:GetValue()].remain ~= nil) then
			remainTime:SetText(tostring(self.db.global.buffsRaidCheck[buffsPick:GetValue()].remain));
		else
			remainTime:SetText("0");
		end
		
		if(self.db.global.buffsRaidCheck[buffsPick:GetValue()].spellNames ~= nil) then
			spellNames:SetText(tostring(self.db.global.buffsRaidCheck[buffsPick:GetValue()].spellNames));
		else
			spellNames:SetText("");
		end
		
		if(self.db.global.buffsRaidCheck[buffsPick:GetValue()].spellIds ~= nil) then
			spellIdEdit:SetText(tostring(self.db.global.buffsRaidCheck[buffsPick:GetValue()].spellIds));
		else
			spellIdEdit:SetText("");
		end
		
		if(self.db.global.buffsRaidCheck[buffsPick:GetValue()].caster ~= nil) then
			classCasterPick:SetValue(classes[self.db.global.buffsRaidCheck[buffsPick:GetValue()].caster]);
		else
			classCasterPick:SetValue(nil);
		end
		
		MorthalinRaidTools:SetPickedClassesRC();
	else
		buffName:SetText("");
		remainTime:SetText("");
		spellNames:SetText("");
		spellIdEdit:SetText("");
		classCasterPick:SetValue(nil);
		classPick:SetValue(nil);
	end
	MorthalinRaidTools:LoadIncludes();
	MorthalinRaidTools:LoadExcludes();
end

function MorthalinRaidTools:SetPickedClassesRC()
	local uniqueClasses = {};
	for i = 1, #self.db.global.buffsRaidCheck[buffsPick:GetValue()].classes do
		uniqueClasses[self.db.global.buffsRaidCheck[buffsPick:GetValue()].classes[i]] = self.db.global.buffsRaidCheck[buffsPick:GetValue()].classes[i];
	end
	local text;
	for i, widget in classPick.pullout:IterateItems() do
		if widget.type == "Dropdown-Item-Toggle" then
			if(uniqueClasses[widget:GetText():sub(11)] ~= nil) then
				widget.value = true;
				widget.check:Show();
				if text then
					text = text..", "..widget:GetText();
				else
					text = widget:GetText();
				end
			else
				widget.value = false;
				widget.check:Hide();
			end
		end
	end
	classPick:SetText(text);
end

function MorthalinRaidTools:GetPickedCasterRC()
	if(classCasterPick:GetValue() == nil or classCasterPick:GetValue() == 10) then
		self.db.global.buffsRaidCheck[buffsPick:GetValue()].caster = "";
	else
		self.db.global.buffsRaidCheck[buffsPick:GetValue()].caster = classCasterPick.list[classCasterPick:GetValue()]:sub(11);
	end
end

function MorthalinRaidTools:LoadIncludes()
	includeName:SetText(nil);
	includePick:SetValue(nil);
	includePick:SetList({});
	if(buffsPick:GetValue() ~= nil) then
		for i = 1, #self.db.global.buffsRaidCheck[buffsPick:GetValue()].include do
			includePick:AddItem(i, self.db.global.buffsRaidCheck[buffsPick:GetValue()].include[i]);
		end
	end
end

function MorthalinRaidTools:LoadExcludes()
	excludeName:SetText(nil);
	excludePick:SetValue(nil);
	excludePick:SetList({});
	if(buffsPick:GetValue() ~= nil) then
		for i = 1, #self.db.global.buffsRaidCheck[buffsPick:GetValue()].exclude do
			excludePick:AddItem(i, self.db.global.buffsRaidCheck[buffsPick:GetValue()].exclude[i]);
		end
	end
end

function MorthalinRaidTools:AddBuffRC()
	table.insert(self.db.global.buffsRaidCheck, {name = "New buff", remain = 300, spellNames = "", spellIds = "", caster = "", classes = {}, include = {}, exclude = {}, index = #self.db.global.buffsRaidCheck});
	buffsPick:AddItem(#self.db.global.buffsRaidCheck, "New buff");
	buffName:SetText("New buff");
	remainTime:SetText("300");
	spellNames:SetText("");
	spellIdEdit:SetText("");
	classCasterPick:SetValue(nil);
	local text;
	for i, widget in classPick.pullout:IterateItems() do
		if widget.type == "Dropdown-Item-Toggle" then
			widget.value = true;
			widget.check:Show();
			if text then
				text = text..", "..widget:GetText();
			else
				text = widget:GetText();
			end
		end
	end
	classPick:SetText(text);
	buffsPick:SetValue(#self.db.global.buffsRaidCheck);
	MorthalinRaidTools:GetPickedClassesRC();
end

function MorthalinRaidTools:RemoveBuffRC(index)
	table.remove(self.db.global.buffsRaidCheck, index);
	MorthalinRaidTools:LoadBuffsRC();
end

function MorthalinRaidTools:GetPickedClassesRC()
	local classes = {};
	for i, widget in classPick.pullout:IterateItems() do
		if widget.type == "Dropdown-Item-Toggle" then
			if(widget:GetValue()) then
				table.insert(classes, widget:GetText():sub(11));
			end
		end
	end
	self.db.global.buffsRaidCheck[buffsPick:GetValue()].classes = classes;
end

function MorthalinRaidTools:SetBuffNameRC(text)
	self.db.global.buffsRaidCheck[buffsPick:GetValue()].name = text;
	buffsPick.pullout.items[buffsPick:GetValue()]:SetText(text);
	buffsPick.list[buffsPick:GetValue()] = text;
	buffsPick:SetText(text);
end

function MorthalinRaidTools:SetRemTimeRC(text)
	local digitCheck = text:gsub("%D+", "");
	if(#text == #digitCheck) then
		self.db.global.buffsRaidCheck[buffsPick:GetValue()].remain = tonumber(digitCheck);
	else
		remainTime:SetText(self.db.global.buffsRaidCheck[buffsPick:GetValue()].remain);
	end
end

function MorthalinRaidTools:SetSpellNamesRC(text)
	self.db.global.buffsRaidCheck[buffsPick:GetValue()].spellNames = text;
end

function MorthalinRaidTools:SetSpellIdsRC(text)
	local temp = text:gsub("%s+", "");
	local endString = "";
	for index, data in pairs({strsplit(",", temp);}) do
		local digitCheck = data:gsub("%D+", "");
		if(#data == #digitCheck) then
			endString = endString .. digitCheck .. ",";
		end
	end
	
	if(#endString ~= 0) then
		endString = endString:sub(1,-2);
		self.db.global.buffsRaidCheck[buffsPick:GetValue()].spellIds = endString;
		spellIdEdit:SetText(endString);
	else
		spellIdEdit:SetText("");
	end
end

function MorthalinRaidTools:AddIncludeRC()
	if(buffsPick:GetValue() ~= nil) then
		table.insert(self.db.global.buffsRaidCheck[buffsPick:GetValue()].include, "New include");
		includePick:AddItem(#self.db.global.buffsRaidCheck[buffsPick:GetValue()].include, "New include");
		includeName:SetText("New include");
		includePick:SetValue(#self.db.global.buffsRaidCheck[buffsPick:GetValue()].include);
	end
end

function MorthalinRaidTools:AddExcludeRC()
	if(buffsPick:GetValue() ~= nil) then
		table.insert(self.db.global.buffsRaidCheck[buffsPick:GetValue()].exclude, "New exclude");
		excludePick:AddItem(#self.db.global.buffsRaidCheck[buffsPick:GetValue()].exclude, "New exclude");
		excludeName:SetText("New exclude");
		excludePick:SetValue(#self.db.global.buffsRaidCheck[buffsPick:GetValue()].exclude);
	end
end

function MorthalinRaidTools:RemoveIncludeRC()
	table.remove(self.db.global.buffsRaidCheck[buffsPick:GetValue()].include, includePick:GetValue());
	MorthalinRaidTools:LoadIncludes();
end

function MorthalinRaidTools:RemoveExcludeRC()
	table.remove(self.db.global.buffsRaidCheck[buffsPick:GetValue()].exclude, includePick:GetValue());
	MorthalinRaidTools:LoadExcludes();
end

function MorthalinRaidTools:IncludePickedRC()
	includeName:SetText(self.db.global.buffsRaidCheck[buffsPick:GetValue()].include[includePick:GetValue()]);
end

function MorthalinRaidTools:ExcludePickedRC()
	excludeName:SetText(self.db.global.buffsRaidCheck[buffsPick:GetValue()].exclude[excludePick:GetValue()]);
end

function MorthalinRaidTools:SetIncludeNameRC(text)
	self.db.global.buffsRaidCheck[buffsPick:GetValue()].include[includePick:GetValue()] = text;
	includePick.pullout.items[includePick:GetValue()]:SetText(text);
	includePick.list[includePick:GetValue()] = text;
	includePick:SetText(text);
end

function MorthalinRaidTools:SetExcludeNameRC(text)
	self.db.global.buffsRaidCheck[buffsPick:GetValue()].exclude[excludePick:GetValue()] = text;
	excludePick.pullout.items[excludePick:GetValue()]:SetText(text);
	excludePick.list[excludePick:GetValue()] = text;
	excludePick:SetText(text);
end
